class UpdateHandler:
    def __init__(self, update_type, filters, function, handled_object_type=None):
        self.update_type = update_type  # str of update type
        self.filters = filters  # list of lambda functions
        self.function = function  # function which will be executed if handler is triggered
        self.handled_object_type = handled_object_type

    def is_triggered(self, update):
        """ checks all handler filters """
        if update['type'] != self.update_type:
            return False

        handled_object = update['object']
        for f in self.filters:
            if not f(handled_object):
                return False
        return True

    def handle(self, handled_object):
        self.function(handled_object)

    def notify(self, update, handled_object_type=None):
        if self.is_triggered(update):
            handled_object = update['object']
            if self.handled_object_type:
                handled_object = self.handled_object_type(**handled_object)
            self.handle(handled_object)



class Param(Exception):
    def __init__(self, message):
        """One of the parameters specified was missing or invalid"""
        super().__init__(message)


class MessagesCantChangeInviteLink(Exception):
    def __init__(self, message):
        """You can't change invite link for this chat"""
        super().__init__(message)


class ParamDocAccess(Exception):
    def __init__(self, message):
        """Access to document is denied"""
        super().__init__(message)


class MessagesChatUserNoAccess(Exception):
    def __init__(self, message):
        """You don't have access to this chat"""
        super().__init__(message)


class AccessAlbum(Exception):
    def __init__(self, message):
        """Access denied"""
        super().__init__(message)


class TooMany(Exception):
    def __init__(self, message):
        """Too many requests per second"""
        super().__init__(message)


class Blocked(Exception):
    def __init__(self, message):
        """Content blocked"""
        super().__init__(message)


class StatusNoAudio(Exception):
    def __init__(self, message):
        """User disabled track name broadcast"""
        super().__init__(message)


class PollsAnswerId(Exception):
    def __init__(self, message):
        """Invalid answer id"""
        super().__init__(message)


class Access(Exception):
    def __init__(self, message):
        """Access denied"""
        super().__init__(message)


class MarketRestoreTooLate(Exception):
    def __init__(self, message):
        """Too late for restore"""
        super().__init__(message)


class VideoAlreadyAdded(Exception):
    def __init__(self, message):
        """This video is already added"""
        super().__init__(message)


class WallAdsPublished(Exception):
    def __init__(self, message):
        """Advertisement post was recently added"""
        super().__init__(message)


class AlbumFull(Exception):
    def __init__(self, message):
        """This album is full"""
        super().__init__(message)


class CallbackApiServersLimit(Exception):
    def __init__(self, message):
        """Servers number limit is reached"""
        super().__init__(message)


class CommunitiesCatalogDisabled(Exception):
    def __init__(self, message):
        """Catalog is not available for this user"""
        super().__init__(message)


class ParamUserId(Exception):
    def __init__(self, message):
        """Invalid user id"""
        super().__init__(message)


class AppAuth(Exception):
    def __init__(self, message):
        """Application authorization failed"""
        super().__init__(message)


class WeightedFlood(Exception):
    def __init__(self, message):
        """Permission denied. You have requested too many actions this day. Try later."""
        super().__init__(message)


class Disabled(Exception):
    def __init__(self, message):
        """Application is disabled. Enable your application or use test mode"""
        super().__init__(message)


class MessagesTooLongForwards(Exception):
    def __init__(self, message):
        """Too many forwarded messages"""
        super().__init__(message)


class MessagesCantFwd(Exception):
    def __init__(self, message):
        """Can't forward these messages"""
        super().__init__(message)


class Server(Exception):
    def __init__(self, message):
        """Internal server error"""
        super().__init__(message)


class MarketCommentsClosed(Exception):
    def __init__(self, message):
        """Comments for this market are closed"""
        super().__init__(message)


class AppsSubscriptionNotFound(Exception):
    def __init__(self, message):
        """Subscription not found"""
        super().__init__(message)


class ParamApiId(Exception):
    def __init__(self, message):
        """Invalid application API ID"""
        super().__init__(message)


class MessagesCantSeeInviteLink(Exception):
    def __init__(self, message):
        """You can't see invite link for this chat"""
        super().__init__(message)


class MessagesMemberAccessToGroupDenied(Exception):
    def __init__(self, message):
        """Can't add user to chat, because user has no access to group"""
        super().__init__(message)


class NeedConfirmation(Exception):
    def __init__(self, message):
        """Confirmation required"""
        super().__init__(message)


class FriendsAddNotFound(Exception):
    def __init__(self, message):
        """Cannot add this user to friends as user not found"""
        super().__init__(message)


class VotesPermission(Exception):
    def __init__(self, message):
        """Permission denied. You must enable votes processing in application settings"""
        super().__init__(message)


class MessagesIntentCantUse(Exception):
    def __init__(self, message):
        """Cannot use this intent"""
        super().__init__(message)


class MarketTooManyAlbums(Exception):
    def __init__(self, message):
        """Too many albums"""
        super().__init__(message)


class GroupAppIsNotInstalledInCommunity(Exception):
    def __init__(self, message):
        """Application is not installed in community"""
        super().__init__(message)


class WallAddPost(Exception):
    def __init__(self, message):
        """Access to adding post denied"""
        super().__init__(message)


class WallLinksForbidden(Exception):
    def __init__(self, message):
        """Hyperlinks are forbidden"""
        super().__init__(message)


class AppsAlreadyUnlocked(Exception):
    def __init__(self, message):
        """This achievement is already unlocked"""
        super().__init__(message)


class WallAdsPostLimitReached(Exception):
    def __init__(self, message):
        """Too many ads posts"""
        super().__init__(message)


class ParamPhoto(Exception):
    def __init__(self, message):
        """Invalid photo"""
        super().__init__(message)


class FriendsListId(Exception):
    def __init__(self, message):
        """Invalid list id"""
        super().__init__(message)


class Runtime(Exception):
    def __init__(self, message):
        """Runtime error occurred during code invocation"""
        super().__init__(message)


class PhotoChanged(Exception):
    def __init__(self, message):
        """Original photo was changed"""
        super().__init__(message)


class PrettyCardsCardNotFound(Exception):
    def __init__(self, message):
        """Card not found"""
        super().__init__(message)


class MessagesChatBotFeature(Exception):
    def __init__(self, message):
        """This is a chat bot feature, change this status in settings"""
        super().__init__(message)


class EnabledInTest(Exception):
    def __init__(self, message):
        """In test mode application should be disabled or user should be authorized"""
        super().__init__(message)


class MethodAds(Exception):
    def __init__(self, message):
        """Permission to perform this action is allowed only for standalone and OpenAPI applications"""
        super().__init__(message)


class StoryExpired(Exception):
    def __init__(self, message):
        """Story has already expired"""
        super().__init__(message)


class WallTooManyRecipients(Exception):
    def __init__(self, message):
        """Too many recipients"""
        super().__init__(message)


class MarketItemNotFound(Exception):
    def __init__(self, message):
        """Item not found"""
        super().__init__(message)


class Compile(Exception):
    def __init__(self, message):
        """Unable to compile code"""
        super().__init__(message)


class ParamServer(Exception):
    def __init__(self, message):
        """Invalid server"""
        super().__init__(message)


class UserDeleted(Exception):
    def __init__(self, message):
        """User was deleted or banned"""
        super().__init__(message)


class ActionFailed(Exception):
    def __init__(self, message):
        """Unable to process action"""
        super().__init__(message)


class ParamNoteId(Exception):
    def __init__(self, message):
        """Note not found"""
        super().__init__(message)


class ParamPhotos(Exception):
    def __init__(self, message):
        """Invalid photos"""
        super().__init__(message)


class AccessVideo(Exception):
    def __init__(self, message):
        """Access denied"""
        super().__init__(message)


class ParamGroupId(Exception):
    def __init__(self, message):
        """Invalid group id"""
        super().__init__(message)


class Request(Exception):
    def __init__(self, message):
        """Invalid request"""
        super().__init__(message)


class AccessComment(Exception):
    def __init__(self, message):
        """Access to comment denied"""
        super().__init__(message)


class MessagesChatDisabled(Exception):
    def __init__(self, message):
        """Chat was disabled"""
        super().__init__(message)


class PollsAccess(Exception):
    def __init__(self, message):
        """Access to poll denied"""
        super().__init__(message)


class MessagesChatNotExist(Exception):
    def __init__(self, message):
        """Chat does not exist"""
        super().__init__(message)


class MessagesTooBig(Exception):
    def __init__(self, message):
        """Can't sent this message, because it's too big"""
        super().__init__(message)


class Method(Exception):
    def __init__(self, message):
        """Unknown method passed"""
        super().__init__(message)


class MarketAlbumNotFound(Exception):
    def __init__(self, message):
        """Album not found"""
        super().__init__(message)


class MessagesMessageRequestAlreadySent(Exception):
    def __init__(self, message):
        """Message request already sent"""
        super().__init__(message)


class PrettyCardsCardIsConnectedToPost(Exception):
    def __init__(self, message):
        """Card is connected to post"""
        super().__init__(message)


class MessagesCantDeleteForAll(Exception):
    def __init__(self, message):
        """Can't delete this message for everybody"""
        super().__init__(message)


class MessagesChatNotAdmin(Exception):
    def __init__(self, message):
        """You are not admin of this chat"""
        super().__init__(message)


class VideoCommentsClosed(Exception):
    def __init__(self, message):
        """Comments for this video are closed"""
        super().__init__(message)


class MethodDisabled(Exception):
    def __init__(self, message):
        """This method was disabled"""
        super().__init__(message)


class Captcha(Exception):
    def __init__(self, message):
        """Captcha needed"""
        super().__init__(message)


class PollsPollId(Exception):
    def __init__(self, message):
        """Invalid poll id"""
        super().__init__(message)


class TooManyLists(Exception):
    def __init__(self, message):
        """Too many feed lists"""
        super().__init__(message)


class Signature(Exception):
    def __init__(self, message):
        """Incorrect signature"""
        super().__init__(message)


class Permission(Exception):
    def __init__(self, message):
        """Permission to perform this action is denied"""
        super().__init__(message)


class AdsObjectDeleted(Exception):
    def __init__(self, message):
        """Object deleted"""
        super().__init__(message)


class CommunitiesCategoriesDisabled(Exception):
    def __init__(self, message):
        """Catalog categories are not available for this user"""
        super().__init__(message)


class RateLimit(Exception):
    def __init__(self, message):
        """Rate limit reached"""
        super().__init__(message)


class MobileNotActivated(Exception):
    def __init__(self, message):
        """The mobile number of the user is unknown"""
        super().__init__(message)


class ParamTimestamp(Exception):
    def __init__(self, message):
        """Invalid timestamp"""
        super().__init__(message)


class StoryIncorrectReplyPrivacy(Exception):
    def __init__(self, message):
        """Incorrect reply privacy"""
        super().__init__(message)


class MessagesTooNewPts(Exception):
    def __init__(self, message):
        """Value of ts or pts is too new"""
        super().__init__(message)


class GroupAuth(Exception):
    def __init__(self, message):
        """Group authorization failed"""
        super().__init__(message)


class MessagesChatUserNotInChat(Exception):
    def __init__(self, message):
        """User not found in chat"""
        super().__init__(message)


class AccessNote(Exception):
    def __init__(self, message):
        """Access to note denied"""
        super().__init__(message)


class ParamDocDeleteAccess(Exception):
    def __init__(self, message):
        """Access to document deleting is denied"""
        super().__init__(message)


class GroupNeed2Fa(Exception):
    def __init__(self, message):
        """You need to enable 2FA for this action"""
        super().__init__(message)


class MarketTooManyItems(Exception):
    def __init__(self, message):
        """Too many items"""
        super().__init__(message)


class ParamHash(Exception):
    def __init__(self, message):
        """Invalid hash"""
        super().__init__(message)


class Limits(Exception):
    def __init__(self, message):
        """Out of limits"""
        super().__init__(message)


class MessagesGroupPeerAccess(Exception):
    def __init__(self, message):
        """Your community can't interact with this peer"""
        super().__init__(message)


class ParamTitle(Exception):
    def __init__(self, message):
        """Invalid title"""
        super().__init__(message)


class MessagesTooLongMessage(Exception):
    def __init__(self, message):
        """Message is too long"""
        super().__init__(message)


class GroupHostNeed2Fa(Exception):
    def __init__(self, message):
        """User needs to enable 2FA for this action"""
        super().__init__(message)


class InsufficientFunds(Exception):
    def __init__(self, message):
        """Application has insufficient funds"""
        super().__init__(message)


class MessagesChatUnsupported(Exception):
    def __init__(self, message):
        """Chat not supported"""
        super().__init__(message)


class ParamPageId(Exception):
    def __init__(self, message):
        """Page not found"""
        super().__init__(message)


class AppsSubscriptionInvalidStatus(Exception):
    def __init__(self, message):
        """Subscription is in invalid status"""
        super().__init__(message)


class PrivateProfile(Exception):
    def __init__(self, message):
        """This profile is private"""
        super().__init__(message)


class AdsPartialSuccess(Exception):
    def __init__(self, message):
        """Some part of the request has not been completed"""
        super().__init__(message)


class MessagesKeyboardInvalid(Exception):
    def __init__(self, message):
        """Keyboard format is invalid"""
        super().__init__(message)


class Upload(Exception):
    def __init__(self, message):
        """Upload error"""
        super().__init__(message)


class WallAccessComment(Exception):
    def __init__(self, message):
        """Access to wall's comment denied"""
        super().__init__(message)


class AdsSpecific(Exception):
    def __init__(self, message):
        """Some ads error occured"""
        super().__init__(message)


class Votes(Exception):
    def __init__(self, message):
        """Not enough votes"""
        super().__init__(message)


class MessagesPrivacy(Exception):
    def __init__(self, message):
        """Can't send messages to this user due to their privacy settings"""
        super().__init__(message)


class MessagesTooOldPts(Exception):
    def __init__(self, message):
        """Value of ts or pts is too old"""
        super().__init__(message)


class GroupTooManyAddresses(Exception):
    def __init__(self, message):
        """Too many addresses in club"""
        super().__init__(message)


class ParamPhone(Exception):
    def __init__(self, message):
        """Invalid phone number"""
        super().__init__(message)


class FriendsListLimit(Exception):
    def __init__(self, message):
        """Reached the maximum number of lists"""
        super().__init__(message)


class FriendsAddYourself(Exception):
    def __init__(self, message):
        """Cannot add user himself as friend"""
        super().__init__(message)


class AuthDelay(Exception):
    def __init__(self, message):
        """Processing.. Try later"""
        super().__init__(message)


class GroupTooManyOfficers(Exception):
    def __init__(self, message):
        """Too many officers in club"""
        super().__init__(message)


class WallAccessReplies(Exception):
    def __init__(self, message):
        """Access to post comments denied"""
        super().__init__(message)


class NotFound(Exception):
    def __init__(self, message):
        """Not found"""
        super().__init__(message)


class PrettyCardsTooManyCards(Exception):
    def __init__(self, message):
        """Too many cards"""
        super().__init__(message)


class AccessMarket(Exception):
    def __init__(self, message):
        """Access denied"""
        super().__init__(message)


class AccessMenu(Exception):
    def __init__(self, message):
        """Access to the menu of the user denied"""
        super().__init__(message)


class FriendsAddEnemy(Exception):
    def __init__(self, message):
        """Cannot add this user to friends as you put him on blacklist"""
        super().__init__(message)


class WallAccessPost(Exception):
    def __init__(self, message):
        """Access to wall's post denied"""
        super().__init__(message)


class MessagesDenySend(Exception):
    def __init__(self, message):
        """Can't send messages for users without permission"""
        super().__init__(message)


class AccessPage(Exception):
    def __init__(self, message):
        """Access to page denied"""
        super().__init__(message)


class Auth(Exception):
    def __init__(self, message):
        """User authorization failed"""
        super().__init__(message)


class AccessAudio(Exception):
    def __init__(self, message):
        """Access denied"""
        super().__init__(message)


class FriendsAddInEnemy(Exception):
    def __init__(self, message):
        """Cannot add this user to friends as they have put you on their blacklist"""
        super().__init__(message)


class MessagesContactNotFound(Exception):
    def __init__(self, message):
        """Contact not found"""
        super().__init__(message)


class AlbumsLimit(Exception):
    def __init__(self, message):
        """Albums number limit is reached"""
        super().__init__(message)


class SaveFile(Exception):
    def __init__(self, message):
        """Couldn't save file"""
        super().__init__(message)


class PollsAccessWithoutVote(Exception):
    def __init__(self, message):
        """Access denied, please vote first"""
        super().__init__(message)


class Unknown(Exception):
    def __init__(self, message):
        """Unknown error occurred"""
        super().__init__(message)


class AdsPermission(Exception):
    def __init__(self, message):
        """Permission denied. You have no access to operations specified with given object(s)"""
        super().__init__(message)


class MessagesEditKindDisallowed(Exception):
    def __init__(self, message):
        """Can't edit this kind of message"""
        super().__init__(message)


class MessagesCantPinOneTimeStory(Exception):
    def __init__(self, message):
        """Cannot pin one-time story"""
        super().__init__(message)


class MessagesTooManyPosts(Exception):
    def __init__(self, message):
        """Too many posts in messages"""
        super().__init__(message)


class MarketItemHasBadLinks(Exception):
    def __init__(self, message):
        """Item has bad links in description"""
        super().__init__(message)


class AccessGroups(Exception):
    def __init__(self, message):
        """Access to the groups list is denied due to the user's privacy settings"""
        super().__init__(message)


class GroupNotInClub(Exception):
    def __init__(self, message):
        """User should be in club"""
        super().__init__(message)


class MessagesIntentLimitOverflow(Exception):
    def __init__(self, message):
        """Limits overflow for this intent"""
        super().__init__(message)


class MethodPermission(Exception):
    def __init__(self, message):
        """Permission to perform this action is denied for non-standalone applications"""
        super().__init__(message)


class PhoneAlreadyUsed(Exception):
    def __init__(self, message):
        """This phone number is used by another user"""
        super().__init__(message)


class MessagesUserBlocked(Exception):
    def __init__(self, message):
        """Can't send messages for users from blacklist"""
        super().__init__(message)


class AuthValidation(Exception):
    def __init__(self, message):
        """Validation required"""
        super().__init__(message)


class WallReplyOwnerFlood(Exception):
    def __init__(self, message):
        """Too many replies"""
        super().__init__(message)


class NeedTokenConfirmation(Exception):
    def __init__(self, message):
        """Token confirmation required"""
        super().__init__(message)


class AccessNoteComment(Exception):
    def __init__(self, message):
        """You can't comment this note"""
        super().__init__(message)


class AuthFloodError(Exception):
    def __init__(self, message):
        """Too many auth attempts, try again later"""
        super().__init__(message)


class GroupChangeCreator(Exception):
    def __init__(self, message):
        """Cannot edit creator role"""
        super().__init__(message)


class WallAccessAddReply(Exception):
    def __init__(self, message):
        """Access to status replies denied"""
        super().__init__(message)


class Flood(Exception):
    def __init__(self, message):
        """Flood control"""
        super().__init__(message)


class ParamDocTitle(Exception):
    def __init__(self, message):
        """Invalid document title"""
        super().__init__(message)


class ParamDocId(Exception):
    def __init__(self, message):
        """Invalid document id"""
        super().__init__(message)


class MessagesEditExpired(Exception):
    def __init__(self, message):
        """Can't edit this message, because it's too old"""
        super().__init__(message)


class AuthHttps(Exception):
    def __init__(self, message):
        """HTTP authorization failed"""
        super().__init__(message)


class MarketTooManyItemsInAlbum(Exception):
    def __init__(self, message):
        """Too many items in album"""
        super().__init__(message)


class ParamAlbumId(Exception):
    def __init__(self, message):
        """Invalid album id"""
        super().__init__(message)


class InvalidAddress(Exception):
    def __init__(self, message):
        """Invalid screen name"""
        super().__init__(message)


class MarketItemAlreadyAdded(Exception):
    def __init__(self, message):
        """Item already added to album"""
        super().__init__(message)


class AccessGroup(Exception):
    def __init__(self, message):
        """Access to group denied"""
        super().__init__(message)


exceptions_dict = {
    100: Param,
    931: MessagesCantChangeInviteLink,
    1153: ParamDocAccess,
    917: MessagesChatUserNoAccess,
    200: AccessAlbum,
    6: TooMany,
    19: Blocked,
    221: StatusNoAudio,
    252: PollsAnswerId,
    15: Access,
    1400: MarketRestoreTooLate,
    800: VideoAlreadyAdded,
    219: WallAdsPublished,
    300: AlbumFull,
    2000: CallbackApiServersLimit,
    1310: CommunitiesCatalogDisabled,
    113: ParamUserId,
    28: AppAuth,
    601: WeightedFlood,
    2: Disabled,
    913: MessagesTooLongForwards,
    921: MessagesCantFwd,
    10: Server,
    1401: MarketCommentsClosed,
    1256: AppsSubscriptionNotFound,
    101: ParamApiId,
    919: MessagesCantSeeInviteLink,
    947: MessagesMemberAccessToGroupDenied,
    24: NeedConfirmation,
    177: FriendsAddNotFound,
    500: VotesPermission,
    943: MessagesIntentCantUse,
    1407: MarketTooManyAlbums,
    711: GroupAppIsNotInstalledInCommunity,
    214: WallAddPost,
    222: WallLinksForbidden,
    1251: AppsAlreadyUnlocked,
    224: WallAdsPostLimitReached,
    129: ParamPhoto,
    171: FriendsListId,
    13: Runtime,
    1160: PhotoChanged,
    1900: PrettyCardsCardNotFound,
    912: MessagesChatBotFeature,
    11: EnabledInTest,
    21: MethodAds,
    1600: StoryExpired,
    220: WallTooManyRecipients,
    1403: MarketItemNotFound,
    12: Compile,
    118: ParamServer,
    18: UserDeleted,
    106: ActionFailed,
    180: ParamNoteId,
    122: ParamPhotos,
    204: AccessVideo,
    125: ParamGroupId,
    8: Request,
    183: AccessComment,
    945: MessagesChatDisabled,
    250: PollsAccess,
    927: MessagesChatNotExist,
    910: MessagesTooBig,
    3: Method,
    1402: MarketAlbumNotFound,
    939: MessagesMessageRequestAlreadySent,
    1902: PrettyCardsCardIsConnectedToPost,
    924: MessagesCantDeleteForAll,
    925: MessagesChatNotAdmin,
    801: VideoCommentsClosed,
    23: MethodDisabled,
    14: Captcha,
    251: PollsPollId,
    1170: TooManyLists,
    4: Signature,
    7: Permission,
    629: AdsObjectDeleted,
    1311: CommunitiesCategoriesDisabled,
    29: RateLimit,
    146: MobileNotActivated,
    150: ParamTimestamp,
    1602: StoryIncorrectReplyPrivacy,
    908: MessagesTooNewPts,
    27: GroupAuth,
    935: MessagesChatUserNotInChat,
    181: AccessNote,
    1151: ParamDocDeleteAccess,
    703: GroupNeed2Fa,
    1405: MarketTooManyItems,
    121: ParamHash,
    103: Limits,
    932: MessagesGroupPeerAccess,
    119: ParamTitle,
    914: MessagesTooLongMessage,
    704: GroupHostNeed2Fa,
    147: InsufficientFunds,
    946: MessagesChatUnsupported,
    140: ParamPageId,
    1257: AppsSubscriptionInvalidStatus,
    30: PrivateProfile,
    602: AdsPartialSuccess,
    911: MessagesKeyboardInvalid,
    22: Upload,
    211: WallAccessComment,
    603: AdsSpecific,
    503: Votes,
    902: MessagesPrivacy,
    907: MessagesTooOldPts,
    706: GroupTooManyAddresses,
    1000: ParamPhone,
    173: FriendsListLimit,
    174: FriendsAddYourself,
    1112: AuthDelay,
    702: GroupTooManyOfficers,
    212: WallAccessReplies,
    104: NotFound,
    1901: PrettyCardsTooManyCards,
    205: AccessMarket,
    148: AccessMenu,
    176: FriendsAddEnemy,
    210: WallAccessPost,
    901: MessagesDenySend,
    141: AccessPage,
    5: Auth,
    201: AccessAudio,
    175: FriendsAddInEnemy,
    936: MessagesContactNotFound,
    302: AlbumsLimit,
    105: SaveFile,
    253: PollsAccessWithoutVote,
    1: Unknown,
    600: AdsPermission,
    920: MessagesEditKindDisallowed,
    942: MessagesCantPinOneTimeStory,
    940: MessagesTooManyPosts,
    1408: MarketItemHasBadLinks,
    260: AccessGroups,
    701: GroupNotInClub,
    944: MessagesIntentLimitOverflow,
    20: MethodPermission,
    1004: PhoneAlreadyUsed,
    900: MessagesUserBlocked,
    17: AuthValidation,
    223: WallReplyOwnerFlood,
    25: NeedTokenConfirmation,
    182: AccessNoteComment,
    1105: AuthFloodError,
    700: GroupChangeCreator,
    213: WallAccessAddReply,
    9: Flood,
    1152: ParamDocTitle,
    1150: ParamDocId,
    909: MessagesEditExpired,
    16: AuthHttps,
    1406: MarketTooManyItemsInAlbum,
    114: ParamAlbumId,
    1260: InvalidAddress,
    1404: MarketItemAlreadyAdded,
    203: AccessGroup,
}


def raise_exception(code, message):
    raise exceptions_dict.get(code, Unknown)(message)



from ..methods.Groups import Groups
import requests

import logging

logger = logging.getLogger('easy_vk.bot.longpoll')
logger.setLevel(logging.DEBUG)


class GroupLongpoll:
    def __init__(self, owner_access_token: str, group_id: int, wait: int = 25, ts: int = None):
        self._access_token = owner_access_token
        self._group_id = group_id
        self._wait = wait

        self._session = requests.Session()

        lp = self.get_lp()
        self._key = lp['key']
        self._server = lp['server']
        self._ts = ts or lp['ts']
        logger.debug(f'Initialise GroupLongpoll class with parameters:\n'
                     f'\towner_access_token = {owner_access_token}'
                     f'\tgroup_id = {group_id}'
                     f'\twait = {wait}'
                     f'\tts = {ts}'
                     )

    def get_lp(self):
        lp = Groups(self._session, self._access_token, '5.120', 3).get_long_poll_server(self._group_id)
        logger.debug(f'Got new longpoll: {lp}')
        return lp

    def update_lp(self):
        lp = self.get_lp()
        self._key = lp['key']
        self._server = lp['server']
        self._ts = lp['ts']
        logger.debug(f'Updated new lonpoll with parameters:\n'
                     f'\tself._key = {lp["key"]}\n'
                     f'\tself._server = {lp["server"]}\n'
                     f'\tself._ts = {lp["ts"]}')

    def get_updates(self, ts_update=True):
        logger.debug('Getting new updates...')
        url = self._server
        params = {
            'act': 'a_check',
            'key': self._key,
            'ts': self._ts,
            'wait': self._wait
        }
        response = self._session.get(url, params=params).json()

        if 'failed' in response:
            if response['failed'] == 1:
                self._ts = response['ts']
                logger.debug(f'Got "fail" = 1 when getting updates. Changing self._ts = {response["ts"]}')
            elif response['failed'] == 2:
                logger.debug(f'Got "fail" = 2 when getting updates. Updating lonpoll...')
                self.update_lp()
            elif response['failed'] == 3:
                logger.debug(f'Got "fail" = 2 when getting updates. Updating lonpoll...')
                self.update_lp()
            return self.get_updates(ts_update)

        if ts_update and response:
            self._ts = response['ts']
            logger.debug(f'Changed ts: self._ts = {response["ts"]}')

        logger.debug(f'Got updates: {response["updates"]}')
        return response['updates']

    def listen(self):
        logger.debug(f'Started listening')
        while True:
            updates = self.get_updates(ts_update=True)
            yield updates



from ._ApiMethod import ApiMethod
from typing import List


class Account(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def ban(self, owner_id: int = None):
        """
        
        :param owner_id: 
        """
    
        method_name = 'account.ban'
        return self._call(method_name, locals())

    def change_password(self, new_password: str, restore_sid: str = None, change_password_hash: str = None, old_password: str = None):
        """
        Changes a user password after access is successfully restored with the [vk.com/dev/auth.restore|auth.restore] method.
        
        :param new_password: New password that will be set as a current
        :param restore_sid: Session id received after the [vk.com/dev/auth.restore|auth.restore] method is executed. (If the password is changed right after the access was restored)
        :param change_password_hash: Hash received after a successful OAuth authorization with a code got by SMS. (If the password is changed right after the access was restored)
        :param old_password: Current user password.
        """
    
        method_name = 'account.changePassword'
        return self._call(method_name, locals())

    def get_active_offers(self, offset: int = None, count: int = None):
        """
        Returns a list of active ads (offers) which executed by the user will bring him/her respective number of votes to his balance in the application.
        
        :param offset: 
        :param count: Number of results to return.
        """
    
        method_name = 'account.getActiveOffers'
        return self._call(method_name, locals())

    def get_app_permissions(self, user_id: int):
        """
        Gets settings of the user in this application.
        
        :param user_id: User ID whose settings information shall be got. By default: current user.
        """
    
        method_name = 'account.getAppPermissions'
        return self._call(method_name, locals())

    def get_banned(self, offset: int = None, count: int = None):
        """
        Returns a user's blacklist.
        
        :param offset: Offset needed to return a specific subset of results.
        :param count: Number of results to return.
        """
    
        method_name = 'account.getBanned'
        return self._call(method_name, locals())

    def get_counters(self, filter_: List[str] = None):
        """
        Returns non-null values of user counters.
        
        :param filter_: Counters to be returned.
        """
    
        if filter_:
            filter_ = ','.join(filter_)
        
        param_alias_dict = {'filter_': 'filter'}
        method_name = 'account.getCounters'
        return self._call(method_name, locals())

    def get_info(self, fields: List[str] = None):
        """
        Returns current account info.
        
        :param fields: Fields to return. Possible values: *'country' — user country,, *'https_required' — is "HTTPS only" option enabled,, *'own_posts_default' — is "Show my posts only" option is enabled,, *'no_wall_replies' — are wall replies disabled or not,, *'intro' — is intro passed by user or not,, *'lang' — user language. By default: all.
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'account.getInfo'
        return self._call(method_name, locals())

    def get_profile_info(self):
        """
        Returns the current account info.
        
        """
    
        method_name = 'account.getProfileInfo'
        return self._call(method_name, locals())

    def get_push_settings(self, device_id: str = None):
        """
        Gets settings of push notifications.
        
        :param device_id: Unique device ID.
        """
    
        method_name = 'account.getPushSettings'
        return self._call(method_name, locals())

    def register_device(self, token: str, device_id: str, device_model: str = None, device_year: int = None, system_version: str = None, settings: str = None, sandbox: bool = None):
        """
        Subscribes an iOS/Android/Windows Phone-based device to receive push notifications
        
        :param token: Device token used to send notifications. (for mpns, the token shall be URL for sending of notifications)
        :param device_id: Unique device ID.
        :param device_model: String name of device model.
        :param device_year: Device year.
        :param system_version: String version of device operating system.
        :param settings: Push settings in a [vk.com/dev/push_settings|special format].
        :param sandbox: 
        """
    
        method_name = 'account.registerDevice'
        return self._call(method_name, locals())

    def save_profile_info(self, first_name: str = None, last_name: str = None, maiden_name: str = None, screen_name: str = None, cancel_request_id: int = None, sex: int = None, relation: int = None, relation_partner_id: int = None, bdate: str = None, bdate_visibility: int = None, home_town: str = None, country_id: int = None, city_id: int = None, status: str = None):
        """
        Edits current profile info.
        
        :param first_name: User first name.
        :param last_name: User last name.
        :param maiden_name: User maiden name (female only)
        :param screen_name: User screen name.
        :param cancel_request_id: ID of the name change request to be canceled. If this parameter is sent, all the others are ignored.
        :param sex: User sex. Possible values: , * '1' – female,, * '2' – male.
        :param relation: User relationship status. Possible values: , * '1' – single,, * '2' – in a relationship,, * '3' – engaged,, * '4' – married,, * '5' – it's complicated,, * '6' – actively searching,, * '7' – in love,, * '0' – not specified.
        :param relation_partner_id: ID of the relationship partner.
        :param bdate: User birth date, format: DD.MM.YYYY.
        :param bdate_visibility: Birth date visibility. Returned values: , * '1' – show birth date,, * '2' – show only month and day,, * '0' – hide birth date.
        :param home_town: User home town.
        :param country_id: User country.
        :param city_id: User city.
        :param status: Status text.
        """
    
        method_name = 'account.saveProfileInfo'
        return self._call(method_name, locals())

    def set_info(self, name: str = None, value: str = None):
        """
        Allows to edit the current account info.
        
        :param name: Setting name.
        :param value: Setting value.
        """
    
        method_name = 'account.setInfo'
        return self._call(method_name, locals())

    def set_name_in_menu(self, user_id: int, name: str = None):
        """
        Sets an application screen name (up to 17 characters), that is shown to the user in the left menu.
        
        :param user_id: User ID.
        :param name: Application screen name.
        """
    
        method_name = 'account.setNameInMenu'
        return self._call(method_name, locals())

    def set_offline(self):
        """
        Marks a current user as offline.
        
        """
    
        method_name = 'account.setOffline'
        return self._call(method_name, locals())

    def set_online(self, voip: bool = None):
        """
        Marks the current user as online for 15 minutes.
        
        :param voip: '1' if videocalls are available for current device.
        """
    
        method_name = 'account.setOnline'
        return self._call(method_name, locals())

    def set_push_settings(self, device_id: str, settings: str = None, key: str = None, value: List[str] = None):
        """
        Change push settings.
        
        :param device_id: Unique device ID.
        :param settings: Push settings in a [vk.com/dev/push_settings|special format].
        :param key: Notification key.
        :param value: New value for the key in a [vk.com/dev/push_settings|special format].
        """
    
        if value:
            value = ','.join(value)
        
        method_name = 'account.setPushSettings'
        return self._call(method_name, locals())

    def set_silence_mode(self, device_id: str = None, time: int = None, peer_id: int = None, sound: int = None):
        """
        Mutes push notifications for the set period of time.
        
        :param device_id: Unique device ID.
        :param time: Time in seconds for what notifications should be disabled. '-1' to disable forever.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'Chat ID', e.g. '2000000001'. For community: '- Community ID', e.g. '-12345'. "
        :param sound: '1' — to enable sound in this dialog, '0' — to disable sound. Only if 'peer_id' contains user or community ID.
        """
    
        method_name = 'account.setSilenceMode'
        return self._call(method_name, locals())

    def unban(self, owner_id: int = None):
        """
        
        :param owner_id: 
        """
    
        method_name = 'account.unban'
        return self._call(method_name, locals())

    def unregister_device(self, device_id: str = None, sandbox: bool = None):
        """
        Unsubscribes a device from push notifications.
        
        :param device_id: Unique device ID.
        :param sandbox: 
        """
    
        method_name = 'account.unregisterDevice'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Database(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def get_chairs(self, faculty_id: int, offset: int = None, count: int = None):
        """
        Returns list of chairs on a specified faculty.
        
        :param faculty_id: id of the faculty to get chairs from
        :param offset: offset required to get a certain subset of chairs
        :param count: amount of chairs to get
        """
    
        method_name = 'database.getChairs'
        return self._call(method_name, locals())

    def get_cities(self, country_id: int, region_id: int = None, q: str = None, need_all: bool = None, offset: int = None, count: int = None):
        """
        Returns a list of cities.
        
        :param country_id: Country ID.
        :param region_id: Region ID.
        :param q: Search query.
        :param need_all: '1' — to return all cities in the country, '0' — to return major cities in the country (default),
        :param offset: Offset needed to return a specific subset of cities.
        :param count: Number of cities to return.
        """
    
        method_name = 'database.getCities'
        return self._call(method_name, locals())

    def get_cities_by_id(self, city_ids: List[int] = None):
        """
        Returns information about cities by their IDs.
        
        :param city_ids: City IDs.
        """
    
        if city_ids:
            city_ids = [str(c) for c in city_ids]
            city_ids = ','.join(city_ids)
        
        method_name = 'database.getCitiesById'
        return self._call(method_name, locals())

    def get_countries(self, need_all: bool = None, code: str = None, offset: int = None, count: int = None):
        """
        Returns a list of countries.
        
        :param need_all: '1' — to return a full list of all countries, '0' — to return a list of countries near the current user's country (default).
        :param code: Country codes in [vk.com/dev/country_codes|ISO 3166-1 alpha-2] standard.
        :param offset: Offset needed to return a specific subset of countries.
        :param count: Number of countries to return.
        """
    
        method_name = 'database.getCountries'
        return self._call(method_name, locals())

    def get_countries_by_id(self, country_ids: List[int] = None):
        """
        Returns information about countries by their IDs.
        
        :param country_ids: Country IDs.
        """
    
        if country_ids:
            country_ids = [str(c) for c in country_ids]
            country_ids = ','.join(country_ids)
        
        method_name = 'database.getCountriesById'
        return self._call(method_name, locals())

    def get_faculties(self, university_id: int, offset: int = None, count: int = None):
        """
        Returns a list of faculties (i.e., university departments).
        
        :param university_id: University ID.
        :param offset: Offset needed to return a specific subset of faculties.
        :param count: Number of faculties to return.
        """
    
        method_name = 'database.getFaculties'
        return self._call(method_name, locals())

    def get_metro_stations(self, city_id: int, offset: int = None, count: int = None, extended: bool = None):
        """
        Get metro stations by city
        
        :param city_id: 
        :param offset: 
        :param count: 
        :param extended: 
        """
    
        method_name = 'database.getMetroStations'
        return self._call(method_name, locals())

    def get_metro_stations_by_id(self, station_ids: List[int] = None):
        """
        Get metro station by his id
        
        :param station_ids: 
        """
    
        if station_ids:
            station_ids = [str(s) for s in station_ids]
            station_ids = ','.join(station_ids)
        
        method_name = 'database.getMetroStationsById'
        return self._call(method_name, locals())

    def get_regions(self, country_id: int, q: str = None, offset: int = None, count: int = None):
        """
        Returns a list of regions.
        
        :param country_id: Country ID, received in [vk.com/dev/database.getCountries|database.getCountries] method.
        :param q: Search query.
        :param offset: Offset needed to return specific subset of regions.
        :param count: Number of regions to return.
        """
    
        method_name = 'database.getRegions'
        return self._call(method_name, locals())

    def get_school_classes(self, country_id: int = None):
        """
        Returns a list of school classes specified for the country.
        
        :param country_id: Country ID.
        """
    
        method_name = 'database.getSchoolClasses'
        return self._call(method_name, locals())

    def get_schools(self, city_id: int, q: str = None, offset: int = None, count: int = None):
        """
        Returns a list of schools.
        
        :param city_id: City ID.
        :param q: Search query.
        :param offset: Offset needed to return a specific subset of schools.
        :param count: Number of schools to return.
        """
    
        method_name = 'database.getSchools'
        return self._call(method_name, locals())

    def get_universities(self, q: str = None, country_id: int = None, city_id: int = None, offset: int = None, count: int = None):
        """
        Returns a list of higher education institutions.
        
        :param q: Search query.
        :param country_id: Country ID.
        :param city_id: City ID.
        :param offset: Offset needed to return a specific subset of universities.
        :param count: Number of universities to return.
        """
    
        method_name = 'database.getUniversities'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Pages(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def clear_cache(self, url: str):
        """
        Allows to clear the cache of particular 'external' pages which may be attached to VK posts.
        
        :param url: Address of the page where you need to refesh the cached version
        """
    
        method_name = 'pages.clearCache'
        return self._call(method_name, locals())

    def get(self, owner_id: int = None, page_id: int = None, global_: bool = None, site_preview: bool = None, title: str = None, need_source: bool = None, need_html: bool = None):
        """
        Returns information about a wiki page.
        
        :param owner_id: Page owner ID.
        :param page_id: Wiki page ID.
        :param global_: '1' — to return information about a global wiki page
        :param site_preview: '1' — resulting wiki page is a preview for the attached link
        :param title: Wiki page title.
        :param need_source: 
        :param need_html: '1' — to return the page as HTML,
        """
    
        param_alias_dict = {'global_': 'global'}
        method_name = 'pages.get'
        return self._call(method_name, locals())

    def get_history(self, page_id: int, group_id: int = None, user_id: int = None):
        """
        Returns a list of all previous versions of a wiki page.
        
        :param page_id: Wiki page ID.
        :param group_id: ID of the community that owns the wiki page.
        :param user_id: 
        """
    
        method_name = 'pages.getHistory'
        return self._call(method_name, locals())

    def get_titles(self, group_id: int = None):
        """
        Returns a list of wiki pages in a group.
        
        :param group_id: ID of the community that owns the wiki page.
        """
    
        method_name = 'pages.getTitles'
        return self._call(method_name, locals())

    def get_version(self, version_id: int, group_id: int = None, user_id: int = None, need_html: bool = None):
        """
        Returns the text of one of the previous versions of a wiki page.
        
        :param version_id: 
        :param group_id: ID of the community that owns the wiki page.
        :param user_id: 
        :param need_html: '1' — to return the page as HTML
        """
    
        method_name = 'pages.getVersion'
        return self._call(method_name, locals())

    def parse_wiki(self, text: str, group_id: int = None):
        """
        Returns HTML representation of the wiki markup.
        
        :param text: Text of the wiki page.
        :param group_id: ID of the group in the context of which this markup is interpreted.
        """
    
        method_name = 'pages.parseWiki'
        return self._call(method_name, locals())

    def save(self, text: str = None, page_id: int = None, group_id: int = None, user_id: int = None, title: str = None):
        """
        Saves the text of a wiki page.
        
        :param text: Text of the wiki page in wiki-format.
        :param page_id: Wiki page ID. The 'title' parameter can be passed instead of 'pid'.
        :param group_id: ID of the community that owns the wiki page.
        :param user_id: User ID
        :param title: Wiki page title.
        """
    
        method_name = 'pages.save'
        return self._call(method_name, locals())

    def save_access(self, page_id: int, group_id: int = None, user_id: int = None, view: int = None, edit: int = None):
        """
        Saves modified read and edit access settings for a wiki page.
        
        :param page_id: Wiki page ID.
        :param group_id: ID of the community that owns the wiki page.
        :param user_id: 
        :param view: Who can view the wiki page: '1' — only community members, '2' — all users can view the page, '0' — only community managers
        :param edit: Who can edit the wiki page: '1' — only community members, '2' — all users can edit the page, '0' — only community managers
        """
    
        method_name = 'pages.saveAccess'
        return self._call(method_name, locals())




from pydantic import BaseModel, Field
from typing import List, Union, Optional, Dict


class Place(BaseModel):
    id: int = Field(None)
    title: str = Field(None)
    latitude: float = Field(None)
    longitude: float = Field(None)
    created: int = Field(None)
    icon: str = Field(None)
    country: str = Field(None)
    city: str = Field(None)


class Geo(BaseModel):
    type: str = Field(None)
    coordinates: List[float] = Field(None)
    place: Place = Field(None)


class MessageAction(BaseModel):
    type: str = Field(None)
    member_id: int = Field(None)
    text: str = Field(None)
    email: str = Field(None)
    photo: Dict[str, str] = Field(None)


class PhotoSize(BaseModel):
    url: str = Field(None)
    width: int = Field(None)
    height: int = Field(None)
    type: str = Field(None)


class Photo(BaseModel):
    id: int = Field(None)
    album_id: int = Field(None)
    owner_id: int = Field(None)
    user_id: int = Field(None)
    text: str = Field(None)
    date: int = Field(None)
    sizes: List[PhotoSize] = Field(None)
    width: int = Field(None)
    height: int = Field(None)


class Video(BaseModel):
    id: int = Field(None)
    owner_id: int = Field(None)
    title: str = Field(None)
    description: str = Field(None)
    duration: int = Field(None)
    photo_130: str = Field(None)
    photo_320: str = Field(None)
    photo_640: str = Field(None)
    photo_800: str = Field(None)
    photo_1280: str = Field(None)
    first_frame_130: str = Field(None)
    first_frame_320: str = Field(None)
    first_frame_640: str = Field(None)
    first_frame_800: str = Field(None)
    first_frame_1280: str = Field(None)
    date: int = Field(None)
    adding_date: int = Field(None)
    views: int = Field(None)
    comments: int = Field(None)
    player: str = Field(None)
    platform: str = Field(None)
    can_edit: int = Field(None)
    can_add: int = Field(None)
    is_private: int = Field(None)
    access_key: str = Field(None)
    processing: int = Field(None)
    live: int = Field(None)
    upcoming: int = Field(None)
    is_favorite: bool = Field(None)


class Audio(BaseModel):
    id: int = Field(None)
    owner_id: int = Field(None)
    artist: str = Field(None)
    title: str = Field(None)
    duration: int = Field(None)
    url: str = Field(None)
    lyrics_id: int = Field(None)
    album_id: int = Field(None)
    genre_id: int = Field(None)
    date: int = Field(None)
    no_search: int = Field(None)
    is_hq: int = Field(None)


class Attachment(BaseModel):
    type: str = Field(None)
    photo: Photo = Field(None)
    audio: Audio = Field(None)
    video: Video = Field(None)


class ButtonAction(BaseModel):
    type: str = Field(...)
    label: str = Field(None)
    payload: str = Field(None)
    link: str = Field(None)
    hash: str = Field(None)
    app_id: int = Field(None)
    owner_id: int = Field(None)


class Button(BaseModel):
    action: ButtonAction = Field(...)
    color: str = Field(None)


class Keyboard(BaseModel):
    one_time: bool = Field(None)
    buttons: List[List[Button]] = Field(None)
    inline: bool = Field(None)


# class MessageClientInfo(BaseModel):
    # button_actions: list


class MessageMessage(BaseModel):
    id: int = Field(None)
    date: int = Field(None)
    peer_id: int = Field(None)
    from_id: int = Field(None)
    text: str = Field(None)
    random_id: int = Field(None)
    ref: str = Field(None)
    ref_source: str = Field(None)
    attachments: List[Attachment] = Field(None)
    important: bool = Field(None)
    geo: Geo = Field(None)
    payload: str = Field(None)
    # keyboard = Field(None)
    fwd_messages: List["MessageMessage"] = Field(None)
    reply_message: "MessageMessage" = Field(None)
    action: Union[MessageAction, str] = Field(None)
    chat_id: int = Field(None)
    chat_active: List[int] = Field(None)
    push_settings: dict = Field(None)
    users_count: int = Field(None)
    admin_id: int = Field(None)
    action_mid: int = Field(None)
    action_email: str = Field(None)
    action_text: str = Field(None)
    photo_50: str = Field(None)
    photo_100: str = Field(None)
    photo_200: str = Field(None)


MessageMessage.update_forward_refs()


class BotMessage(BaseModel):
    message: MessageMessage = Field(None)
    # client_info: MessageClientInfo = Field(None)



from typing import List

import requests
import re
import logging

from .longpoll import GroupLongpoll
from .handlers import UpdateHandler
from ..objects.objects import BotMessage
from ..methods import Messages

logger = logging.getLogger('easy_vk.bot')
logger.setLevel(logging.DEBUG)


class GroupBot:
    def __init__(self, owner_access_token: str,
                 group_access_token: str,
                 group_id: int,
                 wait: int = 25,
                 debug_mode: bool = False,
                 owner_id: int = None):
        self._owner_access_token = owner_access_token
        self._group_access_token = group_access_token
        self._group_id = group_id
        self._wait = wait
        self._debug_mode = debug_mode
        self._owner_id = owner_id

        self._handlers: List[UpdateHandler] = []

        self.longpoll = GroupLongpoll(owner_access_token, group_id, wait)

        # api methods
        session = requests.Session()
        self.messages = Messages(session, group_access_token, '5.120', 25)

    ############################# Handlers
    def message_typing_state_handler(self, user_id: int = None,
                                     user_ids: List[int] = None):
        def decorator(function):
            update_type = 'message_typing_state'
            filters = []
            if user_id:
                filters.append(lambda x: x['from_id'] == user_id)
            if user_ids:
                filters.append(lambda x: x['from_id'] in user_ids)

            handler = UpdateHandler(update_type, filters, function)
            self._handlers.append(handler)
            return function

        return decorator

    def message_new_handler(self, regexp: str = None,
                            user_id: int = None,
                            user_ids: List[int] = None,
                            attachment_type: str = None):
        if self._debug_mode:
            user_id = self._owner_id

        def decorator(function):
            update_type = 'message_new'
            filters = []
            if regexp:
                filters.append(lambda x: True if re.fullmatch(regexp, x.get('text', '')) else None)
            if user_id:
                filters.append(lambda x: x['message']['from_id'] == user_id)
            if user_ids:
                filters.append(lambda x: x['message']['from_id'] in user_ids)
            if attachment_type:
                filters.append(lambda x: attachment_type in [a['type'] for a in x['message']['attachments']])

            handler = UpdateHandler(update_type, filters, function, handled_object_type=BotMessage)
            self._handlers.append(handler)
            return function

        return decorator

    def group_leave_handler(self, user_id: int = None,
                            user_ids: List[int] = None,
                            self_: bool = None):
        def decorator(function):
            update_type = 'group_leave'
            filters = []
            if user_id:
                filters.append(lambda x: x['user_id'] == user_id)
            if user_ids:
                filters.append(lambda x: x['user_id'] in user_ids)
            if self_:
                filters.append(lambda x: x['self'] == self_)

            handler = UpdateHandler(update_type, filters, function)
            self._handlers.append(handler)
            return function

        return decorator

    def group_join_handler(self, user_id: int = None,
                           user_ids: List[int] = None,
                           join_type: bool = None):
        def decorator(function):
            update_type = 'group_join'
            filters = []
            if user_id:
                filters.append(lambda x: x['user_id'] == user_id)
            if user_ids:
                filters.append(lambda x: x['user_id'] in user_ids)
            if join_type:
                filters.append(lambda x: x['join_type'] == join_type)

            handler = UpdateHandler(update_type, filters, function)
            self._handlers.append(handler)
            return function

        return decorator

    ####################################################

    def _handle(self, update):
        for handler in self._handlers:
            handler.notify(update)

    def run(self):
        logger.debug(f'Run bot for group https://vk.com/public{self._group_id}\n')
        for updates in self.longpoll.listen():
            for update in updates:
                self._handle(update)



from ._ApiMethod import ApiMethod
from typing import List


class Status(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def get(self, user_id: int = None, group_id: int = None):
        """
        Returns data required to show the status of a user or community.
        
        :param user_id: User ID or community ID. Use a negative value to designate a community ID.
        :param group_id: 
        """
    
        method_name = 'status.get'
        return self._call(method_name, locals())

    def set(self, text: str = None, group_id: int = None):
        """
        Sets a new status for the current user.
        
        :param text: Text of the new status.
        :param group_id: Identifier of a community to set a status in. If left blank the status is set to current user.
        """
    
        method_name = 'status.set'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Leads(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def check_user(self, lead_id: int, test_result: int = None, test_mode: bool = None, auto_start: bool = None, age: int = None, country: str = None):
        """
        Checks if the user can start the lead.
        
        :param lead_id: Lead ID.
        :param test_result: Value to be return in 'result' field when test mode is used.
        :param test_mode: 
        :param auto_start: 
        :param age: User age.
        :param country: User country code.
        """
    
        method_name = 'leads.checkUser'
        return self._call(method_name, locals())

    def complete(self, vk_sid: str, secret: str, comment: str = None):
        """
        Completes the lead started by user.
        
        :param vk_sid: Session obtained as GET parameter when session started.
        :param secret: Secret key from the lead testing interface.
        :param comment: Comment text.
        """
    
        method_name = 'leads.complete'
        return self._call(method_name, locals())

    def get_stats(self, lead_id: int, secret: str = None, date_start: str = None, date_end: str = None):
        """
        Returns lead stats data.
        
        :param lead_id: Lead ID.
        :param secret: Secret key obtained from the lead testing interface.
        :param date_start: Day to start stats from (YYYY_MM_DD, e.g.2011-09-17).
        :param date_end: Day to finish stats (YYYY_MM_DD, e.g.2011-09-17).
        """
    
        method_name = 'leads.getStats'
        return self._call(method_name, locals())

    def get_users(self, offer_id: int, secret: str, offset: int = None, count: int = None, status: int = None, reverse: bool = None):
        """
        Returns a list of last user actions for the offer.
        
        :param offer_id: Offer ID.
        :param secret: Secret key obtained in the lead testing interface.
        :param offset: Offset needed to return a specific subset of results.
        :param count: Number of results to return.
        :param status: Action type. Possible values: *'0' — start,, *'1' — finish,, *'2' — blocking users,, *'3' — start in a test mode,, *'4' — finish in a test mode.
        :param reverse: Sort order. Possible values: *'1' — chronological,, *'0' — reverse chronological.
        """
    
        method_name = 'leads.getUsers'
        return self._call(method_name, locals())

    def metric_hit(self, data: str):
        """
        Counts the metric event.
        
        :param data: Metric data obtained in the lead interface.
        """
    
        method_name = 'leads.metricHit'
        return self._call(method_name, locals())

    def start(self, lead_id: int, secret: str, uid: int = None, aid: int = None, test_mode: bool = None, force: bool = None):
        """
        Creates new session for the user passing the offer.
        
        :param lead_id: Lead ID.
        :param secret: Secret key from the lead testing interface.
        :param uid: 
        :param aid: 
        :param test_mode: 
        :param force: 
        """
    
        method_name = 'leads.start'
        return self._call(method_name, locals())




from . import methods
from requests import Session


class User:
    def __init__(self, access_token):
        requests_per_s = 3
        v = '5.120'
        session = Session()

        self._access_token = access_token
        self._v = v
        self._requests_per_s = requests_per_s
        self._session = session

        self.ads = methods.Ads(session, access_token, v, requests_per_s)
        self.stories = methods.Stories(session, access_token, v, requests_per_s)
        self.video = methods.Video(session, access_token, v, requests_per_s)
        self.docs = methods.Docs(session, access_token, v, requests_per_s)
        self.photos = methods.Photos(session, access_token, v, requests_per_s)
        self.notes = methods.Notes(session, access_token, v, requests_per_s)
        self.widgets = methods.Widgets(session, access_token, v, requests_per_s)
        self.market = methods.Market(session, access_token, v, requests_per_s)
        self.wall = methods.Wall(session, access_token, v, requests_per_s)
        self.newsfeed = methods.Newsfeed(session, access_token, v, requests_per_s)
        self.fave = methods.Fave(session, access_token, v, requests_per_s)
        self.account = methods.Account(session, access_token, v, requests_per_s)
        self.auth = methods.Auth(session, access_token, v, requests_per_s)
        self.groups = methods.Groups(session, access_token, v, requests_per_s)
        self.status = methods.Status(session, access_token, v, requests_per_s)
        self.appwidgets = methods.Appwidgets(session, access_token, v, requests_per_s)
        self.polls = methods.Polls(session, access_token, v, requests_per_s)
        self.utils = methods.Utils(session, access_token, v, requests_per_s)
        self.messages = methods.Messages(session, access_token, v, requests_per_s)
        self.apps = methods.Apps(session, access_token, v, requests_per_s)
        self.orders = methods.Orders(session, access_token, v, requests_per_s)
        self.pages = methods.Pages(session, access_token, v, requests_per_s)
        self.search = methods.Search(session, access_token, v, requests_per_s)
        self.notifications = methods.Notifications(session, access_token, v, requests_per_s)
        self.stats = methods.Stats(session, access_token, v, requests_per_s)
        self.board = methods.Board(session, access_token, v, requests_per_s)
        self.secure = methods.Secure(session, access_token, v, requests_per_s)
        self.leads = methods.Leads(session, access_token, v, requests_per_s)
        self.streaming = methods.Streaming(session, access_token, v, requests_per_s)
        self.likes = methods.Likes(session, access_token, v, requests_per_s)
        self.prettycards = methods.Prettycards(session, access_token, v, requests_per_s)
        self.users = methods.Users(session, access_token, v, requests_per_s)
        self.friends = methods.Friends(session, access_token, v, requests_per_s)
        self.database = methods.Database(session, access_token, v, requests_per_s)
        self.storage = methods.Storage(session, access_token, v, requests_per_s)
        self.gifts = methods.Gifts(session, access_token, v, requests_per_s)
        self.execute = methods.Execute(session, access_token, v, requests_per_s).execute



from ._ApiMethod import ApiMethod
from typing import List


class Photos(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def confirm_tag(self, photo_id: str, tag_id: int, owner_id: int = None):
        """
        Confirms a tag on a photo.
        
        :param photo_id: Photo ID.
        :param tag_id: Tag ID.
        :param owner_id: ID of the user or community that owns the photo.
        """
    
        method_name = 'photos.confirmTag'
        return self._call(method_name, locals())

    def copy(self, owner_id: int, photo_id: int, access_key: str = None):
        """
        Allows to copy a photo to the "Saved photos" album
        
        :param owner_id: photo's owner ID
        :param photo_id: photo ID
        :param access_key: for private photos
        """
    
        method_name = 'photos.copy'
        return self._call(method_name, locals())

    def create_album(self, title: str, group_id: int = None, description: str = None, privacy_view: List[str] = None, privacy_comment: List[str] = None, upload_by_admins_only: bool = None, comments_disabled: bool = None):
        """
        Creates an empty photo album.
        
        :param title: Album title.
        :param group_id: ID of the community in which the album will be created.
        :param description: Album description.
        :param privacy_view: 
        :param privacy_comment: 
        :param upload_by_admins_only: 
        :param comments_disabled: 
        """
    
        if privacy_view:
            privacy_view = ','.join(privacy_view)
        if privacy_comment:
            privacy_comment = ','.join(privacy_comment)
        
        method_name = 'photos.createAlbum'
        return self._call(method_name, locals())

    def create_comment(self, photo_id: int, owner_id: int = None, message: str = None, attachments: List[str] = None, from_group: bool = None, reply_to_comment: int = None, sticker_id: int = None, access_key: str = None, guid: str = None):
        """
        Adds a new comment on the photo.
        
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        :param message: Comment text.
        :param attachments: (Required if 'message' is not set.) List of objects attached to the post, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, '<owner_id>' — Media attachment owner ID. '<media_id>' — Media attachment ID. Example: "photo100172_166443618,photo66748_265827614"
        :param from_group: '1' — to post a comment from the community
        :param reply_to_comment: 
        :param sticker_id: 
        :param access_key: 
        :param guid: 
        """
    
        if attachments:
            attachments = ','.join(attachments)
        
        method_name = 'photos.createComment'
        return self._call(method_name, locals())

    def delete(self, photo_id: int, owner_id: int = None):
        """
        Deletes a photo.
        
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        """
    
        method_name = 'photos.delete'
        return self._call(method_name, locals())

    def delete_album(self, album_id: int, group_id: int = None):
        """
        Deletes a photo album belonging to the current user.
        
        :param album_id: Album ID.
        :param group_id: ID of the community that owns the album.
        """
    
        method_name = 'photos.deleteAlbum'
        return self._call(method_name, locals())

    def delete_comment(self, comment_id: int, owner_id: int = None):
        """
        Deletes a comment on the photo.
        
        :param comment_id: Comment ID.
        :param owner_id: ID of the user or community that owns the photo.
        """
    
        method_name = 'photos.deleteComment'
        return self._call(method_name, locals())

    def edit(self, photo_id: int, owner_id: int = None, caption: str = None, latitude: float = None, longitude: float = None, place_str: str = None, foursquare_id: str = None, delete_place: bool = None):
        """
        Edits the caption of a photo.
        
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        :param caption: New caption for the photo. If this parameter is not set, it is considered to be equal to an empty string.
        :param latitude: 
        :param longitude: 
        :param place_str: 
        :param foursquare_id: 
        :param delete_place: 
        """
    
        method_name = 'photos.edit'
        return self._call(method_name, locals())

    def edit_album(self, album_id: int, title: str = None, description: str = None, owner_id: int = None, privacy_view: List[str] = None, privacy_comment: List[str] = None, upload_by_admins_only: bool = None, comments_disabled: bool = None):
        """
        Edits information about a photo album.
        
        :param album_id: ID of the photo album to be edited.
        :param title: New album title.
        :param description: New album description.
        :param owner_id: ID of the user or community that owns the album.
        :param privacy_view: 
        :param privacy_comment: 
        :param upload_by_admins_only: 
        :param comments_disabled: 
        """
    
        if privacy_view:
            privacy_view = ','.join(privacy_view)
        if privacy_comment:
            privacy_comment = ','.join(privacy_comment)
        
        method_name = 'photos.editAlbum'
        return self._call(method_name, locals())

    def edit_comment(self, comment_id: int, owner_id: int = None, message: str = None, attachments: List[str] = None):
        """
        Edits a comment on a photo.
        
        :param comment_id: Comment ID.
        :param owner_id: ID of the user or community that owns the photo.
        :param message: New text of the comment.
        :param attachments: (Required if 'message' is not set.) List of objects attached to the post, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, '<owner_id>' — Media attachment owner ID. '<media_id>' — Media attachment ID. Example: "photo100172_166443618,photo66748_265827614"
        """
    
        if attachments:
            attachments = ','.join(attachments)
        
        method_name = 'photos.editComment'
        return self._call(method_name, locals())

    def get(self, owner_id: int = None, album_id: str = None, photo_ids: List[str] = None, rev: bool = None, extended: bool = None, feed_type: str = None, feed: int = None, photo_sizes: bool = None, offset: int = None, count: int = None):
        """
        Returns a list of a user's or community's photos.
        
        :param owner_id: ID of the user or community that owns the photos. Use a negative value to designate a community ID.
        :param album_id: Photo album ID. To return information about photos from service albums, use the following string values: 'profile, wall, saved'.
        :param photo_ids: Photo IDs.
        :param rev: Sort order: '1' — reverse chronological, '0' — chronological
        :param extended: '1' — to return additional 'likes', 'comments', and 'tags' fields, '0' — (default)
        :param feed_type: Type of feed obtained in 'feed' field of the method.
        :param feed: unixtime, that can be obtained with [vk.com/dev/newsfeed.get|newsfeed.get] method in date field to get all photos uploaded by the user on a specific day, or photos the user has been tagged on. Also, 'uid' parameter of the user the event happened with shall be specified.
        :param photo_sizes: '1' — to return photo sizes in a [vk.com/dev/photo_sizes|special format]
        :param offset: 
        :param count: 
        """
    
        if photo_ids:
            photo_ids = ','.join(photo_ids)
        
        method_name = 'photos.get'
        return self._call(method_name, locals())

    def get_albums(self, owner_id: int = None, album_ids: List[int] = None, offset: int = None, count: int = None, need_system: bool = None, need_covers: bool = None, photo_sizes: bool = None):
        """
        Returns a list of a user's or community's photo albums.
        
        :param owner_id: ID of the user or community that owns the albums.
        :param album_ids: Album IDs.
        :param offset: Offset needed to return a specific subset of albums.
        :param count: Number of albums to return.
        :param need_system: '1' — to return system albums with negative IDs
        :param need_covers: '1' — to return an additional 'thumb_src' field, '0' — (default)
        :param photo_sizes: '1' — to return photo sizes in a
        """
    
        if album_ids:
            album_ids = [str(a) for a in album_ids]
            album_ids = ','.join(album_ids)
        
        method_name = 'photos.getAlbums'
        return self._call(method_name, locals())

    def get_albums_count(self, user_id: int = None, group_id: int = None):
        """
        Returns the number of photo albums belonging to a user or community.
        
        :param user_id: User ID.
        :param group_id: Community ID.
        """
    
        method_name = 'photos.getAlbumsCount'
        return self._call(method_name, locals())

    def get_all(self, owner_id: int = None, extended: bool = None, offset: int = None, count: int = None, photo_sizes: bool = None, no_service_albums: bool = None, need_hidden: bool = None, skip_hidden: bool = None):
        """
        Returns a list of photos belonging to a user or community, in reverse chronological order.
        
        :param owner_id: ID of a user or community that owns the photos. Use a negative value to designate a community ID.
        :param extended: '1' — to return detailed information about photos
        :param offset: Offset needed to return a specific subset of photos. By default, '0'.
        :param count: Number of photos to return.
        :param photo_sizes: '1' – to return image sizes in [vk.com/dev/photo_sizes|special format].
        :param no_service_albums: '1' – to return photos only from standard albums, '0' – to return all photos including those in service albums, e.g., 'My wall photos' (default)
        :param need_hidden: '1' – to show information about photos being hidden from the block above the wall.
        :param skip_hidden: '1' – not to return photos being hidden from the block above the wall. Works only with owner_id>0, no_service_albums is ignored.
        """
    
        method_name = 'photos.getAll'
        return self._call(method_name, locals())

    def get_all_comments(self, owner_id: int = None, album_id: int = None, need_likes: bool = None, offset: int = None, count: int = None):
        """
        Returns a list of comments on a specific photo album or all albums of the user sorted in reverse chronological order.
        
        :param owner_id: ID of the user or community that owns the album(s).
        :param album_id: Album ID. If the parameter is not set, comments on all of the user's albums will be returned.
        :param need_likes: '1' — to return an additional 'likes' field, '0' — (default)
        :param offset: Offset needed to return a specific subset of comments. By default, '0'.
        :param count: Number of comments to return. By default, '20'. Maximum value, '100'.
        """
    
        method_name = 'photos.getAllComments'
        return self._call(method_name, locals())

    def get_by_id(self, photos: List[str], extended: bool = None, photo_sizes: bool = None):
        """
        Returns information about photos by their IDs.
        
        :param photos: IDs separated with a comma, that are IDs of users who posted photos and IDs of photos themselves with an underscore character between such IDs. To get information about a photo in the group album, you shall specify group ID instead of user ID. Example: "1_129207899,6492_135055734, , -20629724_271945303"
        :param extended: '1' — to return additional fields, '0' — (default)
        :param photo_sizes: '1' — to return photo sizes in a
        """
    
        if photos:
            photos = ','.join(photos)
        
        method_name = 'photos.getById'
        return self._call(method_name, locals())

    def get_chat_upload_server(self, chat_id: int, crop_x: int = None, crop_y: int = None, crop_width: int = None):
        """
        Returns an upload link for chat cover pictures.
        
        :param chat_id: ID of the chat for which you want to upload a cover photo.
        :param crop_x: 
        :param crop_y: 
        :param crop_width: Width (in pixels) of the photo after cropping.
        """
    
        method_name = 'photos.getChatUploadServer'
        return self._call(method_name, locals())

    def get_comments(self, photo_id: int, owner_id: int = None, need_likes: bool = None, start_comment_id: int = None, offset: int = None, count: int = None, sort: str = None, access_key: str = None, extended: bool = None, fields: List[str] = None):
        """
        Returns a list of comments on a photo.
        
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        :param need_likes: '1' — to return an additional 'likes' field, '0' — (default)
        :param start_comment_id: 
        :param offset: Offset needed to return a specific subset of comments. By default, '0'.
        :param count: Number of comments to return.
        :param sort: Sort order: 'asc' — old first, 'desc' — new first
        :param access_key: 
        :param extended: 
        :param fields: 
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'photos.getComments'
        return self._call(method_name, locals())

    def get_market_album_upload_server(self, group_id: int):
        """
        Returns the server address for market album photo upload.
        
        :param group_id: Community ID.
        """
    
        method_name = 'photos.getMarketAlbumUploadServer'
        return self._call(method_name, locals())

    def get_market_upload_server(self, group_id: int, main_photo: bool = None, crop_x: int = None, crop_y: int = None, crop_width: int = None):
        """
        Returns the server address for market photo upload.
        
        :param group_id: Community ID.
        :param main_photo: '1' if you want to upload the main item photo.
        :param crop_x: X coordinate of the crop left upper corner.
        :param crop_y: Y coordinate of the crop left upper corner.
        :param crop_width: Width of the cropped photo in px.
        """
    
        method_name = 'photos.getMarketUploadServer'
        return self._call(method_name, locals())

    def get_messages_upload_server(self, peer_id: int = None):
        """
        Returns the server address for photo upload in a private message for a user.
        
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'Chat ID', e.g. '2000000001'. For community: '- Community ID', e.g. '-12345'. "
        """
    
        method_name = 'photos.getMessagesUploadServer'
        return self._call(method_name, locals())

    def get_new_tags(self, offset: int = None, count: int = None):
        """
        Returns a list of photos with tags that have not been viewed.
        
        :param offset: Offset needed to return a specific subset of photos.
        :param count: Number of photos to return.
        """
    
        method_name = 'photos.getNewTags'
        return self._call(method_name, locals())

    def get_owner_cover_photo_upload_server(self, group_id: int, crop_x: int = None, crop_y: int = None, crop_x2: int = None, crop_y2: int = None):
        """
        Returns the server address for owner cover upload.
        
        :param group_id: ID of community that owns the album (if the photo will be uploaded to a community album).
        :param crop_x: X coordinate of the left-upper corner
        :param crop_y: Y coordinate of the left-upper corner
        :param crop_x2: X coordinate of the right-bottom corner
        :param crop_y2: Y coordinate of the right-bottom corner
        """
    
        method_name = 'photos.getOwnerCoverPhotoUploadServer'
        return self._call(method_name, locals())

    def get_owner_photo_upload_server(self, owner_id: int = None):
        """
        Returns an upload server address for a profile or community photo.
        
        :param owner_id: identifier of a community or current user. "Note that community id must be negative. 'owner_id=1' – user, 'owner_id=-1' – community, "
        """
    
        method_name = 'photos.getOwnerPhotoUploadServer'
        return self._call(method_name, locals())

    def get_tags(self, photo_id: int, owner_id: int = None, access_key: str = None):
        """
        Returns a list of tags on a photo.
        
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        :param access_key: 
        """
    
        method_name = 'photos.getTags'
        return self._call(method_name, locals())

    def get_upload_server(self, group_id: int = None, album_id: int = None):
        """
        Returns the server address for photo upload.
        
        :param group_id: ID of community that owns the album (if the photo will be uploaded to a community album).
        :param album_id: 
        """
    
        method_name = 'photos.getUploadServer'
        return self._call(method_name, locals())

    def get_user_photos(self, user_id: int = None, offset: int = None, count: int = None, extended: bool = None, sort: str = None):
        """
        Returns a list of photos in which a user is tagged.
        
        :param user_id: User ID.
        :param offset: Offset needed to return a specific subset of photos. By default, '0'.
        :param count: Number of photos to return. Maximum value is 1000.
        :param extended: '1' — to return an additional 'likes' field, '0' — (default)
        :param sort: Sort order: '1' — by date the tag was added in ascending order, '0' — by date the tag was added in descending order
        """
    
        method_name = 'photos.getUserPhotos'
        return self._call(method_name, locals())

    def get_wall_upload_server(self, group_id: int = None):
        """
        Returns the server address for photo upload onto a user's wall.
        
        :param group_id: ID of community to whose wall the photo will be uploaded.
        """
    
        method_name = 'photos.getWallUploadServer'
        return self._call(method_name, locals())

    def make_cover(self, photo_id: int, owner_id: int = None, album_id: int = None):
        """
        Makes a photo into an album cover.
        
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        :param album_id: Album ID.
        """
    
        method_name = 'photos.makeCover'
        return self._call(method_name, locals())

    def move(self, target_album_id: int, photo_id: int, owner_id: int = None):
        """
        Moves a photo from one album to another.
        
        :param target_album_id: ID of the album to which the photo will be moved.
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        """
    
        method_name = 'photos.move'
        return self._call(method_name, locals())

    def put_tag(self, photo_id: int, user_id: int, owner_id: int = None, x: float = None, y: float = None, x2: float = None, y2: float = None):
        """
        Adds a tag on the photo.
        
        :param photo_id: Photo ID.
        :param user_id: ID of the user to be tagged.
        :param owner_id: ID of the user or community that owns the photo.
        :param x: Upper left-corner coordinate of the tagged area (as a percentage of the photo's width).
        :param y: Upper left-corner coordinate of the tagged area (as a percentage of the photo's height).
        :param x2: Lower right-corner coordinate of the tagged area (as a percentage of the photo's width).
        :param y2: Lower right-corner coordinate of the tagged area (as a percentage of the photo's height).
        """
    
        method_name = 'photos.putTag'
        return self._call(method_name, locals())

    def remove_tag(self, photo_id: int, tag_id: int, owner_id: int = None):
        """
        Removes a tag from a photo.
        
        :param photo_id: Photo ID.
        :param tag_id: Tag ID.
        :param owner_id: ID of the user or community that owns the photo.
        """
    
        method_name = 'photos.removeTag'
        return self._call(method_name, locals())

    def reorder_albums(self, album_id: int, owner_id: int = None, before: int = None, after: int = None):
        """
        Reorders the album in the list of user albums.
        
        :param album_id: Album ID.
        :param owner_id: ID of the user or community that owns the album.
        :param before: ID of the album before which the album in question shall be placed.
        :param after: ID of the album after which the album in question shall be placed.
        """
    
        method_name = 'photos.reorderAlbums'
        return self._call(method_name, locals())

    def reorder_photos(self, photo_id: int, owner_id: int = None, before: int = None, after: int = None):
        """
        Reorders the photo in the list of photos of the user album.
        
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        :param before: ID of the photo before which the photo in question shall be placed.
        :param after: ID of the photo after which the photo in question shall be placed.
        """
    
        method_name = 'photos.reorderPhotos'
        return self._call(method_name, locals())

    def report(self, owner_id: int, photo_id: int, reason: int = None):
        """
        Reports (submits a complaint about) a photo.
        
        :param owner_id: ID of the user or community that owns the photo.
        :param photo_id: Photo ID.
        :param reason: Reason for the complaint: '0' – spam, '1' – child pornography, '2' – extremism, '3' – violence, '4' – drug propaganda, '5' – adult material, '6' – insult, abuse
        """
    
        method_name = 'photos.report'
        return self._call(method_name, locals())

    def report_comment(self, owner_id: int, comment_id: int, reason: int = None):
        """
        Reports (submits a complaint about) a comment on a photo.
        
        :param owner_id: ID of the user or community that owns the photo.
        :param comment_id: ID of the comment being reported.
        :param reason: Reason for the complaint: '0' – spam, '1' – child pornography, '2' – extremism, '3' – violence, '4' – drug propaganda, '5' – adult material, '6' – insult, abuse
        """
    
        method_name = 'photos.reportComment'
        return self._call(method_name, locals())

    def restore(self, photo_id: int, owner_id: int = None):
        """
        Restores a deleted photo.
        
        :param photo_id: Photo ID.
        :param owner_id: ID of the user or community that owns the photo.
        """
    
        method_name = 'photos.restore'
        return self._call(method_name, locals())

    def restore_comment(self, comment_id: int, owner_id: int = None):
        """
        Restores a deleted comment on a photo.
        
        :param comment_id: ID of the deleted comment.
        :param owner_id: ID of the user or community that owns the photo.
        """
    
        method_name = 'photos.restoreComment'
        return self._call(method_name, locals())

    def save(self, album_id: int = None, group_id: int = None, server: int = None, photos_list: str = None, hash_: str = None, latitude: float = None, longitude: float = None, caption: str = None):
        """
        Saves photos after successful uploading.
        
        :param album_id: ID of the album to save photos to.
        :param group_id: ID of the community to save photos to.
        :param server: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param photos_list: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param hash_: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param latitude: Geographical latitude, in degrees (from '-90' to '90').
        :param longitude: Geographical longitude, in degrees (from '-180' to '180').
        :param caption: Text describing the photo. 2048 digits max.
        """
    
        param_alias_dict = {'hash_': 'hash'}
        method_name = 'photos.save'
        return self._call(method_name, locals())

    def save_market_album_photo(self, group_id: int, photo: str, server: int, hash_: str):
        """
        Saves market album photos after successful uploading.
        
        :param group_id: Community ID.
        :param photo: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param server: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param hash_: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        """
    
        param_alias_dict = {'hash_': 'hash'}
        method_name = 'photos.saveMarketAlbumPhoto'
        return self._call(method_name, locals())

    def save_market_photo(self, photo: str, server: int, hash_: str, group_id: int = None, crop_data: str = None, crop_hash: str = None):
        """
        Saves market photos after successful uploading.
        
        :param photo: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param server: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param hash_: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param group_id: Community ID.
        :param crop_data: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param crop_hash: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        """
    
        param_alias_dict = {'hash_': 'hash'}
        method_name = 'photos.saveMarketPhoto'
        return self._call(method_name, locals())

    def save_messages_photo(self, photo: str, server: int = None, hash_: str = None):
        """
        Saves a photo after being successfully uploaded. URL obtained with [vk.com/dev/photos.getMessagesUploadServer|photos.getMessagesUploadServer] method.
        
        :param photo: Parameter returned when the photo is [vk.com/dev/upload_files|uploaded to the server].
        :param server: 
        :param hash_: 
        """
    
        param_alias_dict = {'hash_': 'hash'}
        method_name = 'photos.saveMessagesPhoto'
        return self._call(method_name, locals())

    def save_owner_cover_photo(self, hash_: str, photo: str):
        """
        Saves cover photo after successful uploading.
        
        :param hash_: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        :param photo: Parameter returned when photos are [vk.com/dev/upload_files|uploaded to server].
        """
    
        param_alias_dict = {'hash_': 'hash'}
        method_name = 'photos.saveOwnerCoverPhoto'
        return self._call(method_name, locals())

    def save_owner_photo(self, server: str = None, hash_: str = None, photo: str = None):
        """
        Saves a profile or community photo. Upload URL can be got with the [vk.com/dev/photos.getOwnerPhotoUploadServer|photos.getOwnerPhotoUploadServer] method.
        
        :param server: parameter returned after [vk.com/dev/upload_files|photo upload].
        :param hash_: parameter returned after [vk.com/dev/upload_files|photo upload].
        :param photo: parameter returned after [vk.com/dev/upload_files|photo upload].
        """
    
        param_alias_dict = {'hash_': 'hash'}
        method_name = 'photos.saveOwnerPhoto'
        return self._call(method_name, locals())

    def save_wall_photo(self, photo: str, user_id: int = None, group_id: int = None, server: int = None, hash_: str = None, latitude: float = None, longitude: float = None, caption: str = None):
        """
        Saves a photo to a user's or community's wall after being uploaded.
        
        :param photo: Parameter returned when the the photo is [vk.com/dev/upload_files|uploaded to the server].
        :param user_id: ID of the user on whose wall the photo will be saved.
        :param group_id: ID of community on whose wall the photo will be saved.
        :param server: 
        :param hash_: 
        :param latitude: Geographical latitude, in degrees (from '-90' to '90').
        :param longitude: Geographical longitude, in degrees (from '-180' to '180').
        :param caption: Text describing the photo. 2048 digits max.
        """
    
        param_alias_dict = {'hash_': 'hash'}
        method_name = 'photos.saveWallPhoto'
        return self._call(method_name, locals())

    def search(self, q: str = None, lat: float = None, long: float = None, start_time: int = None, end_time: int = None, sort: int = None, offset: int = None, count: int = None, radius: int = None):
        """
        Returns a list of photos.
        
        :param q: Search query string.
        :param lat: Geographical latitude, in degrees (from '-90' to '90').
        :param long: Geographical longitude, in degrees (from '-180' to '180').
        :param start_time: 
        :param end_time: 
        :param sort: Sort order:
        :param offset: Offset needed to return a specific subset of photos.
        :param count: Number of photos to return.
        :param radius: Radius of search in meters (works very approximately). Available values: '10', '100', '800', '6000', '50000'.
        """
    
        method_name = 'photos.search'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Storage(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def get(self, key: str = None, keys: List[str] = None, user_id: int = None):
        """
        Returns a value of variable with the name set by key parameter.
        
        :param key: 
        :param keys: 
        :param user_id: 
        """
    
        if keys:
            keys = ','.join(keys)
        
        method_name = 'storage.get'
        return self._call(method_name, locals())

    def get_keys(self, user_id: int = None, offset: int = None, count: int = None):
        """
        Returns the names of all variables.
        
        :param user_id: user id, whose variables names are returned if they were requested with a server method.
        :param offset: 
        :param count: amount of variable names the info needs to be collected from.
        """
    
        method_name = 'storage.getKeys'
        return self._call(method_name, locals())

    def set(self, key: str, value: str = None, user_id: int = None):
        """
        Saves a value of variable with the name set by 'key' parameter.
        
        :param key: 
        :param value: 
        :param user_id: 
        """
    
        method_name = 'storage.set'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Groups(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def add_address(self, group_id: int, title: str, address: str, country_id: int, city_id: int, latitude: float, longitude: float, additional_address: str = None, metro_id: int = None, phone: str = None, work_info_status: str = None, timetable: str = None, is_main_address: bool = None):
        """
        
        :param group_id: 
        :param title: 
        :param address: 
        :param country_id: 
        :param city_id: 
        :param latitude: 
        :param longitude: 
        :param additional_address: 
        :param metro_id: 
        :param phone: 
        :param work_info_status: 
        :param timetable: 
        :param is_main_address: 
        """
    
        method_name = 'groups.addAddress'
        return self._call(method_name, locals())

    def add_callback_server(self, group_id: int, url: str, title: str, secret_key: str = None):
        """
        
        :param group_id: 
        :param url: 
        :param title: 
        :param secret_key: 
        """
    
        method_name = 'groups.addCallbackServer'
        return self._call(method_name, locals())

    def add_link(self, group_id: int, link: str, text: str = None):
        """
        Allows to add a link to the community.
        
        :param group_id: Community ID.
        :param link: Link URL.
        :param text: Description text for the link.
        """
    
        method_name = 'groups.addLink'
        return self._call(method_name, locals())

    def approve_request(self, group_id: int, user_id: int):
        """
        Allows to approve join request to the community.
        
        :param group_id: Community ID.
        :param user_id: User ID.
        """
    
        method_name = 'groups.approveRequest'
        return self._call(method_name, locals())

    def ban(self, group_id: int, owner_id: int = None, end_date: int = None, reason: int = None, comment: str = None, comment_visible: bool = None):
        """
        
        :param group_id: 
        :param owner_id: 
        :param end_date: 
        :param reason: 
        :param comment: 
        :param comment_visible: 
        """
    
        method_name = 'groups.ban'
        return self._call(method_name, locals())

    def create(self, title: str, description: str = None, type_: str = None, public_category: int = None, subtype: int = None):
        """
        Creates a new community.
        
        :param title: Community title.
        :param description: Community description (ignored for 'type' = 'public').
        :param type_: Community type. Possible values: *'group' – group,, *'event' – event,, *'public' – public page
        :param public_category: Category ID (for 'type' = 'public' only).
        :param subtype: Public page subtype. Possible values: *'1' – place or small business,, *'2' – company, organization or website,, *'3' – famous person or group of people,, *'4' – product or work of art.
        """
    
        param_alias_dict = {'type_': 'type'}
        method_name = 'groups.create'
        return self._call(method_name, locals())

    def delete_callback_server(self, group_id: int, server_id: int):
        """
        
        :param group_id: 
        :param server_id: 
        """
    
        method_name = 'groups.deleteCallbackServer'
        return self._call(method_name, locals())

    def delete_link(self, group_id: int, link_id: int):
        """
        Allows to delete a link from the community.
        
        :param group_id: Community ID.
        :param link_id: Link ID.
        """
    
        method_name = 'groups.deleteLink'
        return self._call(method_name, locals())

    def disable_online(self, group_id: int):
        """
        
        :param group_id: 
        """
    
        method_name = 'groups.disableOnline'
        return self._call(method_name, locals())

    def edit(self, group_id: int, title: str = None, description: str = None, screen_name: str = None, access: int = None, website: str = None, subject: str = None, email: str = None, phone: str = None, rss: str = None, event_start_date: int = None, event_finish_date: int = None, event_group_id: int = None, public_category: int = None, public_subcategory: int = None, public_date: str = None, wall: int = None, topics: int = None, photos: int = None, video: int = None, audio: int = None, links: bool = None, events: bool = None, places: bool = None, contacts: bool = None, docs: int = None, wiki: int = None, messages: bool = None, articles: bool = None, addresses: bool = None, age_limits: int = None, market: bool = None, market_comments: bool = None, market_country: List[int] = None, market_city: List[int] = None, market_currency: int = None, market_contact: int = None, market_wiki: int = None, obscene_filter: bool = None, obscene_stopwords: bool = None, obscene_words: List[str] = None, main_section: int = None, secondary_section: int = None, country: int = None, city: int = None):
        """
        Edits a community.
        
        :param group_id: Community ID.
        :param title: Community title.
        :param description: Community description.
        :param screen_name: Community screen name.
        :param access: Community type. Possible values: *'0' – open,, *'1' – closed,, *'2' – private.
        :param website: Website that will be displayed in the community information field.
        :param subject: Community subject. Possible values: , *'1' – auto/moto,, *'2' – activity holidays,, *'3' – business,, *'4' – pets,, *'5' – health,, *'6' – dating and communication, , *'7' – games,, *'8' – IT (computers and software),, *'9' – cinema,, *'10' – beauty and fashion,, *'11' – cooking,, *'12' – art and culture,, *'13' – literature,, *'14' – mobile services and internet,, *'15' – music,, *'16' – science and technology,, *'17' – real estate,, *'18' – news and media,, *'19' – security,, *'20' – education,, *'21' – home and renovations,, *'22' – politics,, *'23' – food,, *'24' – industry,, *'25' – travel,, *'26' – work,, *'27' – entertainment,, *'28' – religion,, *'29' – family,, *'30' – sports,, *'31' – insurance,, *'32' – television,, *'33' – goods and services,, *'34' – hobbies,, *'35' – finance,, *'36' – photo,, *'37' – esoterics,, *'38' – electronics and appliances,, *'39' – erotic,, *'40' – humor,, *'41' – society, humanities,, *'42' – design and graphics.
        :param email: Organizer email (for events).
        :param phone: Organizer phone number (for events).
        :param rss: RSS feed address for import (available only to communities with special permission. Contact vk.com/support to get it.
        :param event_start_date: Event start date in Unixtime format.
        :param event_finish_date: Event finish date in Unixtime format.
        :param event_group_id: Organizer community ID (for events only).
        :param public_category: Public page category ID.
        :param public_subcategory: Public page subcategory ID.
        :param public_date: Founding date of a company or organization owning the community in "dd.mm.YYYY" format.
        :param wall: Wall settings. Possible values: *'0' – disabled,, *'1' – open,, *'2' – limited (groups and events only),, *'3' – closed (groups and events only).
        :param topics: Board topics settings. Possbile values: , *'0' – disabled,, *'1' – open,, *'2' – limited (for groups and events only).
        :param photos: Photos settings. Possible values: *'0' – disabled,, *'1' – open,, *'2' – limited (for groups and events only).
        :param video: Video settings. Possible values: *'0' – disabled,, *'1' – open,, *'2' – limited (for groups and events only).
        :param audio: Audio settings. Possible values: *'0' – disabled,, *'1' – open,, *'2' – limited (for groups and events only).
        :param links: Links settings (for public pages only). Possible values: *'0' – disabled,, *'1' – enabled.
        :param events: Events settings (for public pages only). Possible values: *'0' – disabled,, *'1' – enabled.
        :param places: Places settings (for public pages only). Possible values: *'0' – disabled,, *'1' – enabled.
        :param contacts: Contacts settings (for public pages only). Possible values: *'0' – disabled,, *'1' – enabled.
        :param docs: Documents settings. Possible values: *'0' – disabled,, *'1' – open,, *'2' – limited (for groups and events only).
        :param wiki: Wiki pages settings. Possible values: *'0' – disabled,, *'1' – open,, *'2' – limited (for groups and events only).
        :param messages: Community messages. Possible values: *'0' — disabled,, *'1' — enabled.
        :param articles: 
        :param addresses: 
        :param age_limits: Community age limits. Possible values: *'1' — no limits,, *'2' — 16+,, *'3' — 18+.
        :param market: Market settings. Possible values: *'0' – disabled,, *'1' – enabled.
        :param market_comments: market comments settings. Possible values: *'0' – disabled,, *'1' – enabled.
        :param market_country: Market delivery countries.
        :param market_city: Market delivery cities (if only one country is specified).
        :param market_currency: Market currency settings. Possbile values: , *'643' – Russian rubles,, *'980' – Ukrainian hryvnia,, *'398' – Kazakh tenge,, *'978' – Euro,, *'840' – US dollars
        :param market_contact: Seller contact for market. Set '0' for community messages.
        :param market_wiki: ID of a wiki page with market description.
        :param obscene_filter: Obscene expressions filter in comments. Possible values: , *'0' – disabled,, *'1' – enabled.
        :param obscene_stopwords: Stopwords filter in comments. Possible values: , *'0' – disabled,, *'1' – enabled.
        :param obscene_words: Keywords for stopwords filter.
        :param main_section: 
        :param secondary_section: 
        :param country: Country of the community.
        :param city: City of the community.
        """
    
        if market_country:
            market_country = [str(m) for m in market_country]
            market_country = ','.join(market_country)
        if market_city:
            market_city = [str(m) for m in market_city]
            market_city = ','.join(market_city)
        if obscene_words:
            obscene_words = ','.join(obscene_words)
        
        method_name = 'groups.edit'
        return self._call(method_name, locals())

    def edit_address(self, group_id: int, address_id: int, title: str = None, address: str = None, additional_address: str = None, country_id: int = None, city_id: int = None, metro_id: int = None, latitude: float = None, longitude: float = None, phone: str = None, work_info_status: str = None, timetable: str = None, is_main_address: bool = None):
        """
        
        :param group_id: 
        :param address_id: 
        :param title: 
        :param address: 
        :param additional_address: 
        :param country_id: 
        :param city_id: 
        :param metro_id: 
        :param latitude: 
        :param longitude: 
        :param phone: 
        :param work_info_status: 
        :param timetable: 
        :param is_main_address: 
        """
    
        method_name = 'groups.editAddress'
        return self._call(method_name, locals())

    def edit_callback_server(self, group_id: int, server_id: int, url: str, title: str, secret_key: str = None):
        """
        
        :param group_id: 
        :param server_id: 
        :param url: 
        :param title: 
        :param secret_key: 
        """
    
        method_name = 'groups.editCallbackServer'
        return self._call(method_name, locals())

    def edit_link(self, group_id: int, link_id: int, text: str = None):
        """
        Allows to edit a link in the community.
        
        :param group_id: Community ID.
        :param link_id: Link ID.
        :param text: New description text for the link.
        """
    
        method_name = 'groups.editLink'
        return self._call(method_name, locals())

    def edit_manager(self, group_id: int, user_id: int, role: str = None, is_contact: bool = None, contact_position: str = None, contact_phone: str = None, contact_email: str = None):
        """
        Allows to add, remove or edit the community manager.
        
        :param group_id: Community ID.
        :param user_id: User ID.
        :param role: Manager role. Possible values: *'moderator',, *'editor',, *'administrator'.
        :param is_contact: '1' — to show the manager in Contacts block of the community.
        :param contact_position: Position to show in Contacts block.
        :param contact_phone: Contact phone.
        :param contact_email: Contact e-mail.
        """
    
        method_name = 'groups.editManager'
        return self._call(method_name, locals())

    def enable_online(self, group_id: int):
        """
        
        :param group_id: 
        """
    
        method_name = 'groups.enableOnline'
        return self._call(method_name, locals())

    def get(self, user_id: int = None, extended: bool = None, filter_: List[str] = None, fields: List[str] = None, offset: int = None, count: int = None):
        """
        Returns a list of the communities to which a user belongs.
        
        :param user_id: User ID.
        :param extended: '1' — to return complete information about a user's communities, '0' — to return a list of community IDs without any additional fields (default),
        :param filter_: Types of communities to return: 'admin' — to return communities administered by the user , 'editor' — to return communities where the user is an administrator or editor, 'moder' — to return communities where the user is an administrator, editor, or moderator, 'groups' — to return only groups, 'publics' — to return only public pages, 'events' — to return only events
        :param fields: Profile fields to return.
        :param offset: Offset needed to return a specific subset of communities.
        :param count: Number of communities to return.
        """
    
        if filter_:
            filter_ = ','.join(filter_)
        if fields:
            fields = ','.join(fields)
        
        param_alias_dict = {'filter_': 'filter'}
        method_name = 'groups.get'
        return self._call(method_name, locals())

    def get_addresses(self, group_id: int, address_ids: List[int] = None, latitude: float = None, longitude: float = None, offset: int = None, count: int = None, fields: List[str] = None):
        """
        Returns a list of community addresses.
        
        :param group_id: ID or screen name of the community.
        :param address_ids: 
        :param latitude: Latitude of  the user geo position.
        :param longitude: Longitude of the user geo position.
        :param offset: Offset needed to return a specific subset of community addresses.
        :param count: Number of community addresses to return.
        :param fields: Address fields
        """
    
        if address_ids:
            address_ids = [str(a) for a in address_ids]
            address_ids = ','.join(address_ids)
        if fields:
            fields = ','.join(fields)
        
        method_name = 'groups.getAddresses'
        return self._call(method_name, locals())

    def get_banned(self, group_id: int, offset: int = None, count: int = None, fields: List[str] = None, owner_id: int = None):
        """
        Returns a list of users on a community blacklist.
        
        :param group_id: Community ID.
        :param offset: Offset needed to return a specific subset of users.
        :param count: Number of users to return.
        :param fields: 
        :param owner_id: 
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'groups.getBanned'
        return self._call(method_name, locals())

    def get_by_id(self, group_ids: List[str] = None, group_id: str = None, fields: List[str] = None):
        """
        Returns information about communities by their IDs.
        
        :param group_ids: IDs or screen names of communities.
        :param group_id: ID or screen name of the community.
        :param fields: Group fields to return.
        """
    
        if group_ids:
            group_ids = ','.join(group_ids)
        if fields:
            fields = ','.join(fields)
        
        method_name = 'groups.getById'
        return self._call(method_name, locals())

    def get_callback_confirmation_code(self, group_id: int):
        """
        Returns Callback API confirmation code for the community.
        
        :param group_id: Community ID.
        """
    
        method_name = 'groups.getCallbackConfirmationCode'
        return self._call(method_name, locals())

    def get_callback_servers(self, group_id: int, server_ids: List[int] = None):
        """
        
        :param group_id: 
        :param server_ids: 
        """
    
        if server_ids:
            server_ids = [str(s) for s in server_ids]
            server_ids = ','.join(server_ids)
        
        method_name = 'groups.getCallbackServers'
        return self._call(method_name, locals())

    def get_callback_settings(self, group_id: int, server_id: int = None):
        """
        Returns [vk.com/dev/callback_api|Callback API] notifications settings.
        
        :param group_id: Community ID.
        :param server_id: Server ID.
        """
    
        method_name = 'groups.getCallbackSettings'
        return self._call(method_name, locals())

    def get_catalog(self, category_id: int = None, subcategory_id: int = None):
        """
        Returns communities list for a catalog category.
        
        :param category_id: Category id received from [vk.com/dev/groups.getCatalogInfo|groups.getCatalogInfo].
        :param subcategory_id: Subcategory id received from [vk.com/dev/groups.getCatalogInfo|groups.getCatalogInfo].
        """
    
        method_name = 'groups.getCatalog'
        return self._call(method_name, locals())

    def get_catalog_info(self, extended: bool = None, subcategories: bool = None):
        """
        Returns categories list for communities catalog
        
        :param extended: 1 – to return communities count and three communities for preview. By default: 0.
        :param subcategories: 1 – to return subcategories info. By default: 0.
        """
    
        method_name = 'groups.getCatalogInfo'
        return self._call(method_name, locals())

    def get_invited_users(self, group_id: int, offset: int = None, count: int = None, fields: List[str] = None, name_case: str = None):
        """
        Returns invited users list of a community
        
        :param group_id: Group ID to return invited users for.
        :param offset: Offset needed to return a specific subset of results.
        :param count: Number of results to return.
        :param fields: List of additional fields to be returned. Available values: 'sex, bdate, city, country, photo_50, photo_100, photo_200_orig, photo_200, photo_400_orig, photo_max, photo_max_orig, online, online_mobile, lists, domain, has_mobile, contacts, connections, site, education, universities, schools, can_post, can_see_all_posts, can_see_audio, can_write_private_message, status, last_seen, common_count, relation, relatives, counters'.
        :param name_case: Case for declension of user name and surname. Possible values: *'nom' — nominative (default),, *'gen' — genitive,, *'dat' — dative,, *'acc' — accusative, , *'ins' — instrumental,, *'abl' — prepositional.
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'groups.getInvitedUsers'
        return self._call(method_name, locals())

    def get_invites(self, offset: int = None, count: int = None, extended: bool = None):
        """
        Returns a list of invitations to join communities and events.
        
        :param offset: Offset needed to return a specific subset of invitations.
        :param count: Number of invitations to return.
        :param extended: '1' — to return additional [vk.com/dev/fields_groups|fields] for communities..
        """
    
        method_name = 'groups.getInvites'
        return self._call(method_name, locals())

    def get_long_poll_server(self, group_id: int):
        """
        Returns the data needed to query a Long Poll server for events
        
        :param group_id: Community ID
        """
    
        method_name = 'groups.getLongPollServer'
        return self._call(method_name, locals())

    def get_long_poll_settings(self, group_id: int):
        """
        Returns Long Poll notification settings
        
        :param group_id: Community ID.
        """
    
        method_name = 'groups.getLongPollSettings'
        return self._call(method_name, locals())

    def get_members(self, group_id: str = None, sort: str = None, offset: int = None, count: int = None, fields: List[str] = None, filter_: str = None):
        """
        Returns a list of community members.
        
        :param group_id: ID or screen name of the community.
        :param sort: Sort order. Available values: 'id_asc', 'id_desc', 'time_asc', 'time_desc'. 'time_asc' and 'time_desc' are availavle only if the method is called by the group's 'moderator'.
        :param offset: Offset needed to return a specific subset of community members.
        :param count: Number of community members to return.
        :param fields: List of additional fields to be returned. Available values: 'sex, bdate, city, country, photo_50, photo_100, photo_200_orig, photo_200, photo_400_orig, photo_max, photo_max_orig, online, online_mobile, lists, domain, has_mobile, contacts, connections, site, education, universities, schools, can_post, can_see_all_posts, can_see_audio, can_write_private_message, status, last_seen, common_count, relation, relatives, counters'.
        :param filter_: *'friends' – only friends in this community will be returned,, *'unsure' – only those who pressed 'I may attend' will be returned (if it's an event).
        """
    
        if fields:
            fields = ','.join(fields)
        
        param_alias_dict = {'filter_': 'filter'}
        method_name = 'groups.getMembers'
        return self._call(method_name, locals())

    def get_requests(self, group_id: int, offset: int = None, count: int = None, fields: List[str] = None):
        """
        Returns a list of requests to the community.
        
        :param group_id: Community ID.
        :param offset: Offset needed to return a specific subset of results.
        :param count: Number of results to return.
        :param fields: Profile fields to return.
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'groups.getRequests'
        return self._call(method_name, locals())

    def get_settings(self, group_id: int):
        """
        Returns community settings.
        
        :param group_id: Community ID.
        """
    
        method_name = 'groups.getSettings'
        return self._call(method_name, locals())

    def get_token_permissions(self):
        """
        
        """
    
        method_name = 'groups.getTokenPermissions'
        return self._call(method_name, locals())

    def invite(self, group_id: int, user_id: int):
        """
        Allows to invite friends to the community.
        
        :param group_id: Community ID.
        :param user_id: User ID.
        """
    
        method_name = 'groups.invite'
        return self._call(method_name, locals())

    def is_member(self, group_id: str, user_id: int = None, user_ids: List[int] = None, extended: bool = None):
        """
        Returns information specifying whether a user is a member of a community.
        
        :param group_id: ID or screen name of the community.
        :param user_id: User ID.
        :param user_ids: User IDs.
        :param extended: '1' — to return an extended response with additional fields. By default: '0'.
        """
    
        if user_ids:
            user_ids = [str(u) for u in user_ids]
            user_ids = ','.join(user_ids)
        
        method_name = 'groups.isMember'
        return self._call(method_name, locals())

    def join(self, group_id: int = None, not_sure: str = None):
        """
        With this method you can join the group or public page, and also confirm your participation in an event.
        
        :param group_id: ID or screen name of the community.
        :param not_sure: Optional parameter which is taken into account when 'gid' belongs to the event: '1' — Perhaps I will attend, '0' — I will be there for sure (default), ,
        """
    
        method_name = 'groups.join'
        return self._call(method_name, locals())

    def leave(self, group_id: int):
        """
        With this method you can leave a group, public page, or event.
        
        :param group_id: ID or screen name of the community.
        """
    
        method_name = 'groups.leave'
        return self._call(method_name, locals())

    def remove_user(self, group_id: int, user_id: int):
        """
        Removes a user from the community.
        
        :param group_id: Community ID.
        :param user_id: User ID.
        """
    
        method_name = 'groups.removeUser'
        return self._call(method_name, locals())

    def reorder_link(self, group_id: int, link_id: int, after: int = None):
        """
        Allows to reorder links in the community.
        
        :param group_id: Community ID.
        :param link_id: Link ID.
        :param after: ID of the link after which to place the link with 'link_id'.
        """
    
        method_name = 'groups.reorderLink'
        return self._call(method_name, locals())

    def search(self, q: str, type_: str = None, country_id: int = None, city_id: int = None, future: bool = None, market: bool = None, sort: int = None, offset: int = None, count: int = None):
        """
        Returns a list of communities matching the search criteria.
        
        :param q: Search query string.
        :param type_: Community type. Possible values: 'group, page, event.'
        :param country_id: Country ID.
        :param city_id: City ID. If this parameter is transmitted, country_id is ignored.
        :param future: '1' — to return only upcoming events. Works with the 'type' = 'event' only.
        :param market: '1' — to return communities with enabled market only.
        :param sort: Sort order. Possible values: *'0' — default sorting (similar the full version of the site),, *'1' — by growth speed,, *'2'— by the "day attendance/members number" ratio,, *'3' — by the "Likes number/members number" ratio,, *'4' — by the "comments number/members number" ratio,, *'5' — by the "boards entries number/members number" ratio.
        :param offset: Offset needed to return a specific subset of results.
        :param count: Number of communities to return. "Note that you can not receive more than first thousand of results, regardless of 'count' and 'offset' values."
        """
    
        param_alias_dict = {'type_': 'type'}
        method_name = 'groups.search'
        return self._call(method_name, locals())

    def set_callback_settings(self, group_id: int, server_id: int = None, api_version: str = None, message_new: bool = None, message_reply: bool = None, message_allow: bool = None, message_edit: bool = None, message_deny: bool = None, message_typing_state: bool = None, photo_new: bool = None, audio_new: bool = None, video_new: bool = None, wall_reply_new: bool = None, wall_reply_edit: bool = None, wall_reply_delete: bool = None, wall_reply_restore: bool = None, wall_post_new: bool = None, wall_repost: bool = None, board_post_new: bool = None, board_post_edit: bool = None, board_post_restore: bool = None, board_post_delete: bool = None, photo_comment_new: bool = None, photo_comment_edit: bool = None, photo_comment_delete: bool = None, photo_comment_restore: bool = None, video_comment_new: bool = None, video_comment_edit: bool = None, video_comment_delete: bool = None, video_comment_restore: bool = None, market_comment_new: bool = None, market_comment_edit: bool = None, market_comment_delete: bool = None, market_comment_restore: bool = None, poll_vote_new: bool = None, group_join: bool = None, group_leave: bool = None, group_change_settings: bool = None, group_change_photo: bool = None, group_officers_edit: bool = None, user_block: bool = None, user_unblock: bool = None, lead_forms_new: bool = None):
        """
        Allow to set notifications settings for group.
        
        :param group_id: Community ID.
        :param server_id: Server ID.
        :param api_version: 
        :param message_new: A new incoming message has been received ('0' — disabled, '1' — enabled).
        :param message_reply: A new outcoming message has been received ('0' — disabled, '1' — enabled).
        :param message_allow: Allowed messages notifications ('0' — disabled, '1' — enabled).
        :param message_edit: 
        :param message_deny: Denied messages notifications ('0' — disabled, '1' — enabled).
        :param message_typing_state: 
        :param photo_new: New photos notifications ('0' — disabled, '1' — enabled).
        :param audio_new: New audios notifications ('0' — disabled, '1' — enabled).
        :param video_new: New videos notifications ('0' — disabled, '1' — enabled).
        :param wall_reply_new: New wall replies notifications ('0' — disabled, '1' — enabled).
        :param wall_reply_edit: Wall replies edited notifications ('0' — disabled, '1' — enabled).
        :param wall_reply_delete: A wall comment has been deleted ('0' — disabled, '1' — enabled).
        :param wall_reply_restore: A wall comment has been restored ('0' — disabled, '1' — enabled).
        :param wall_post_new: New wall posts notifications ('0' — disabled, '1' — enabled).
        :param wall_repost: New wall posts notifications ('0' — disabled, '1' — enabled).
        :param board_post_new: New board posts notifications ('0' — disabled, '1' — enabled).
        :param board_post_edit: Board posts edited notifications ('0' — disabled, '1' — enabled).
        :param board_post_restore: Board posts restored notifications ('0' — disabled, '1' — enabled).
        :param board_post_delete: Board posts deleted notifications ('0' — disabled, '1' — enabled).
        :param photo_comment_new: New comment to photo notifications ('0' — disabled, '1' — enabled).
        :param photo_comment_edit: A photo comment has been edited ('0' — disabled, '1' — enabled).
        :param photo_comment_delete: A photo comment has been deleted ('0' — disabled, '1' — enabled).
        :param photo_comment_restore: A photo comment has been restored ('0' — disabled, '1' — enabled).
        :param video_comment_new: New comment to video notifications ('0' — disabled, '1' — enabled).
        :param video_comment_edit: A video comment has been edited ('0' — disabled, '1' — enabled).
        :param video_comment_delete: A video comment has been deleted ('0' — disabled, '1' — enabled).
        :param video_comment_restore: A video comment has been restored ('0' — disabled, '1' — enabled).
        :param market_comment_new: New comment to market item notifications ('0' — disabled, '1' — enabled).
        :param market_comment_edit: A market comment has been edited ('0' — disabled, '1' — enabled).
        :param market_comment_delete: A market comment has been deleted ('0' — disabled, '1' — enabled).
        :param market_comment_restore: A market comment has been restored ('0' — disabled, '1' — enabled).
        :param poll_vote_new: A vote in a public poll has been added ('0' — disabled, '1' — enabled).
        :param group_join: Joined community notifications ('0' — disabled, '1' — enabled).
        :param group_leave: Left community notifications ('0' — disabled, '1' — enabled).
        :param group_change_settings: 
        :param group_change_photo: 
        :param group_officers_edit: 
        :param user_block: User added to community blacklist
        :param user_unblock: User removed from community blacklist
        :param lead_forms_new: New form in lead forms
        """
    
        method_name = 'groups.setCallbackSettings'
        return self._call(method_name, locals())

    def set_long_poll_settings(self, group_id: int, enabled: bool = None, api_version: str = None, message_new: bool = None, message_reply: bool = None, message_allow: bool = None, message_deny: bool = None, message_edit: bool = None, message_typing_state: bool = None, photo_new: bool = None, audio_new: bool = None, video_new: bool = None, wall_reply_new: bool = None, wall_reply_edit: bool = None, wall_reply_delete: bool = None, wall_reply_restore: bool = None, wall_post_new: bool = None, wall_repost: bool = None, board_post_new: bool = None, board_post_edit: bool = None, board_post_restore: bool = None, board_post_delete: bool = None, photo_comment_new: bool = None, photo_comment_edit: bool = None, photo_comment_delete: bool = None, photo_comment_restore: bool = None, video_comment_new: bool = None, video_comment_edit: bool = None, video_comment_delete: bool = None, video_comment_restore: bool = None, market_comment_new: bool = None, market_comment_edit: bool = None, market_comment_delete: bool = None, market_comment_restore: bool = None, poll_vote_new: bool = None, group_join: bool = None, group_leave: bool = None, group_change_settings: bool = None, group_change_photo: bool = None, group_officers_edit: bool = None, user_block: bool = None, user_unblock: bool = None):
        """
        Sets Long Poll notification settings
        
        :param group_id: Community ID.
        :param enabled: Sets whether Long Poll is enabled ('0' — disabled, '1' — enabled).
        :param api_version: 
        :param message_new: A new incoming message has been received ('0' — disabled, '1' — enabled).
        :param message_reply: A new outcoming message has been received ('0' — disabled, '1' — enabled).
        :param message_allow: Allowed messages notifications ('0' — disabled, '1' — enabled).
        :param message_deny: Denied messages notifications ('0' — disabled, '1' — enabled).
        :param message_edit: A message has been edited ('0' — disabled, '1' — enabled).
        :param message_typing_state: 
        :param photo_new: New photos notifications ('0' — disabled, '1' — enabled).
        :param audio_new: New audios notifications ('0' — disabled, '1' — enabled).
        :param video_new: New videos notifications ('0' — disabled, '1' — enabled).
        :param wall_reply_new: New wall replies notifications ('0' — disabled, '1' — enabled).
        :param wall_reply_edit: Wall replies edited notifications ('0' — disabled, '1' — enabled).
        :param wall_reply_delete: A wall comment has been deleted ('0' — disabled, '1' — enabled).
        :param wall_reply_restore: A wall comment has been restored ('0' — disabled, '1' — enabled).
        :param wall_post_new: New wall posts notifications ('0' — disabled, '1' — enabled).
        :param wall_repost: New wall posts notifications ('0' — disabled, '1' — enabled).
        :param board_post_new: New board posts notifications ('0' — disabled, '1' — enabled).
        :param board_post_edit: Board posts edited notifications ('0' — disabled, '1' — enabled).
        :param board_post_restore: Board posts restored notifications ('0' — disabled, '1' — enabled).
        :param board_post_delete: Board posts deleted notifications ('0' — disabled, '1' — enabled).
        :param photo_comment_new: New comment to photo notifications ('0' — disabled, '1' — enabled).
        :param photo_comment_edit: A photo comment has been edited ('0' — disabled, '1' — enabled).
        :param photo_comment_delete: A photo comment has been deleted ('0' — disabled, '1' — enabled).
        :param photo_comment_restore: A photo comment has been restored ('0' — disabled, '1' — enabled).
        :param video_comment_new: New comment to video notifications ('0' — disabled, '1' — enabled).
        :param video_comment_edit: A video comment has been edited ('0' — disabled, '1' — enabled).
        :param video_comment_delete: A video comment has been deleted ('0' — disabled, '1' — enabled).
        :param video_comment_restore: A video comment has been restored ('0' — disabled, '1' — enabled).
        :param market_comment_new: New comment to market item notifications ('0' — disabled, '1' — enabled).
        :param market_comment_edit: A market comment has been edited ('0' — disabled, '1' — enabled).
        :param market_comment_delete: A market comment has been deleted ('0' — disabled, '1' — enabled).
        :param market_comment_restore: A market comment has been restored ('0' — disabled, '1' — enabled).
        :param poll_vote_new: A vote in a public poll has been added ('0' — disabled, '1' — enabled).
        :param group_join: Joined community notifications ('0' — disabled, '1' — enabled).
        :param group_leave: Left community notifications ('0' — disabled, '1' — enabled).
        :param group_change_settings: 
        :param group_change_photo: 
        :param group_officers_edit: 
        :param user_block: User added to community blacklist
        :param user_unblock: User removed from community blacklist
        """
    
        method_name = 'groups.setLongPollSettings'
        return self._call(method_name, locals())

    def unban(self, group_id: int, owner_id: int = None):
        """
        
        :param group_id: 
        :param owner_id: 
        """
    
        method_name = 'groups.unban'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Docs(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def add(self, owner_id: int, doc_id: int, access_key: str = None):
        """
        Copies a document to a user's or community's document list.
        
        :param owner_id: ID of the user or community that owns the document. Use a negative value to designate a community ID.
        :param doc_id: Document ID.
        :param access_key: Access key. This parameter is required if 'access_key' was returned with the document's data.
        """
    
        method_name = 'docs.add'
        return self._call(method_name, locals())

    def delete(self, owner_id: int, doc_id: int):
        """
        Deletes a user or community document.
        
        :param owner_id: ID of the user or community that owns the document. Use a negative value to designate a community ID.
        :param doc_id: Document ID.
        """
    
        method_name = 'docs.delete'
        return self._call(method_name, locals())

    def edit(self, owner_id: int, doc_id: int, title: str = None, tags: List[str] = None):
        """
        Edits a document.
        
        :param owner_id: User ID or community ID. Use a negative value to designate a community ID.
        :param doc_id: Document ID.
        :param title: Document title.
        :param tags: Document tags.
        """
    
        if tags:
            tags = ','.join(tags)
        
        method_name = 'docs.edit'
        return self._call(method_name, locals())

    def get(self, count: int = None, offset: int = None, type_: int = None, owner_id: int = None, return_tags: bool = None):
        """
        Returns detailed information about user or community documents.
        
        :param count: Number of documents to return. By default, all documents.
        :param offset: Offset needed to return a specific subset of documents.
        :param type_: 
        :param owner_id: ID of the user or community that owns the documents. Use a negative value to designate a community ID.
        :param return_tags: 
        """
    
        param_alias_dict = {'type_': 'type'}
        method_name = 'docs.get'
        return self._call(method_name, locals())

    def get_by_id(self, docs: List[str], return_tags: bool = None):
        """
        Returns information about documents by their IDs.
        
        :param docs: Document IDs. Example: , "66748_91488,66748_91455",
        :param return_tags: 
        """
    
        if docs:
            docs = ','.join(docs)
        
        method_name = 'docs.getById'
        return self._call(method_name, locals())

    def get_messages_upload_server(self, type_: str = None, peer_id: int = None):
        """
        Returns the server address for document upload.
        
        :param type_: Document type.
        :param peer_id: Destination ID. "For user: 'User ID', e.g. '12345'. For chat: '2000000000' + 'Chat ID', e.g. '2000000001'. For community: '- Community ID', e.g. '-12345'. "
        """
    
        param_alias_dict = {'type_': 'type'}
        method_name = 'docs.getMessagesUploadServer'
        return self._call(method_name, locals())

    def get_types(self, owner_id: int):
        """
        Returns documents types available for current user.
        
        :param owner_id: ID of the user or community that owns the documents. Use a negative value to designate a community ID.
        """
    
        method_name = 'docs.getTypes'
        return self._call(method_name, locals())

    def get_upload_server(self, group_id: int = None):
        """
        Returns the server address for document upload.
        
        :param group_id: Community ID (if the document will be uploaded to the community).
        """
    
        method_name = 'docs.getUploadServer'
        return self._call(method_name, locals())

    def get_wall_upload_server(self, group_id: int = None):
        """
        Returns the server address for document upload onto a user's or community's wall.
        
        :param group_id: Community ID (if the document will be uploaded to the community).
        """
    
        method_name = 'docs.getWallUploadServer'
        return self._call(method_name, locals())

    def save(self, file: str, title: str = None, tags: str = None, return_tags: bool = None):
        """
        Saves a document after [vk.com/dev/upload_files_2|uploading it to a server].
        
        :param file: This parameter is returned when the file is [vk.com/dev/upload_files_2|uploaded to the server].
        :param title: Document title.
        :param tags: Document tags.
        :param return_tags: 
        """
    
        method_name = 'docs.save'
        return self._call(method_name, locals())

    def search(self, q: str, search_own: bool = None, count: int = None, offset: int = None, return_tags: bool = None):
        """
        Returns a list of documents matching the search criteria.
        
        :param q: Search query string.
        :param search_own: 
        :param count: Number of results to return.
        :param offset: Offset needed to return a specific subset of results.
        :param return_tags: 
        """
    
        method_name = 'docs.search'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Video(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def add(self, video_id: int, owner_id: int, target_id: int = None):
        """
        Adds a video to a user or community page.
        
        :param video_id: Video ID.
        :param owner_id: ID of the user or community that owns the video. Use a negative value to designate a community ID.
        :param target_id: identifier of a user or community to add a video to. Use a negative value to designate a community ID.
        """
    
        method_name = 'video.add'
        return self._call(method_name, locals())

    def add_album(self, group_id: int = None, title: str = None, privacy: List[str] = None):
        """
        Creates an empty album for videos.
        
        :param group_id: Community ID (if the album will be created in a community).
        :param title: Album title.
        :param privacy: new access permissions for the album. Possible values: , *'0' – all users,, *'1' – friends only,, *'2' – friends and friends of friends,, *'3' – "only me".
        """
    
        if privacy:
            privacy = ','.join(privacy)
        
        method_name = 'video.addAlbum'
        return self._call(method_name, locals())

    def add_to_album(self, owner_id: int, video_id: int, target_id: int = None, album_id: int = None, album_ids: List[int] = None):
        """
        
        :param owner_id: 
        :param video_id: 
        :param target_id: 
        :param album_id: 
        :param album_ids: 
        """
    
        if album_ids:
            album_ids = [str(a) for a in album_ids]
            album_ids = ','.join(album_ids)
        
        method_name = 'video.addToAlbum'
        return self._call(method_name, locals())

    def create_comment(self, video_id: int, owner_id: int = None, message: str = None, attachments: List[str] = None, from_group: bool = None, reply_to_comment: int = None, sticker_id: int = None, guid: str = None):
        """
        Adds a new comment on a video.
        
        :param video_id: Video ID.
        :param owner_id: ID of the user or community that owns the video.
        :param message: New comment text.
        :param attachments: List of objects attached to the comment, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, '<owner_id>' — ID of the media attachment owner. '<media_id>' — Media attachment ID. Example: "photo100172_166443618,photo66748_265827614"
        :param from_group: '1' — to post the comment from a community name (only if 'owner_id'<0)
        :param reply_to_comment: 
        :param sticker_id: 
        :param guid: 
        """
    
        if attachments:
            attachments = ','.join(attachments)
        
        method_name = 'video.createComment'
        return self._call(method_name, locals())

    def delete(self, video_id: int, owner_id: int = None, target_id: int = None):
        """
        Deletes a video from a user or community page.
        
        :param video_id: Video ID.
        :param owner_id: ID of the user or community that owns the video.
        :param target_id: 
        """
    
        method_name = 'video.delete'
        return self._call(method_name, locals())

    def delete_album(self, album_id: int, group_id: int = None):
        """
        Deletes a video album.
        
        :param album_id: Album ID.
        :param group_id: Community ID (if the album is owned by a community).
        """
    
        method_name = 'video.deleteAlbum'
        return self._call(method_name, locals())

    def delete_comment(self, comment_id: int, owner_id: int = None):
        """
        Deletes a comment on a video.
        
        :param comment_id: ID of the comment to be deleted.
        :param owner_id: ID of the user or community that owns the video.
        """
    
        method_name = 'video.deleteComment'
        return self._call(method_name, locals())

    def edit(self, video_id: int, owner_id: int = None, name: str = None, desc: str = None, privacy_view: List[str] = None, privacy_comment: List[str] = None, no_comments: bool = None, repeat: bool = None):
        """
        Edits information about a video on a user or community page.
        
        :param video_id: Video ID.
        :param owner_id: ID of the user or community that owns the video.
        :param name: New video title.
        :param desc: New video description.
        :param privacy_view: Privacy settings in a [vk.com/dev/privacy_setting|special format]. Privacy setting is available for videos uploaded to own profile by user.
        :param privacy_comment: Privacy settings for comments in a [vk.com/dev/privacy_setting|special format].
        :param no_comments: Disable comments for the group video.
        :param repeat: '1' — to repeat the playback of the video, '0' — to play the video once,
        """
    
        if privacy_view:
            privacy_view = ','.join(privacy_view)
        if privacy_comment:
            privacy_comment = ','.join(privacy_comment)
        
        method_name = 'video.edit'
        return self._call(method_name, locals())

    def edit_album(self, album_id: int, title: str, group_id: int = None, privacy: List[str] = None):
        """
        Edits the title of a video album.
        
        :param album_id: Album ID.
        :param title: New album title.
        :param group_id: Community ID (if the album edited is owned by a community).
        :param privacy: new access permissions for the album. Possible values: , *'0' – all users,, *'1' – friends only,, *'2' – friends and friends of friends,, *'3' – "only me".
        """
    
        if privacy:
            privacy = ','.join(privacy)
        
        method_name = 'video.editAlbum'
        return self._call(method_name, locals())

    def edit_comment(self, comment_id: int, owner_id: int = None, message: str = None, attachments: List[str] = None):
        """
        Edits the text of a comment on a video.
        
        :param comment_id: Comment ID.
        :param owner_id: ID of the user or community that owns the video.
        :param message: New comment text.
        :param attachments: List of objects attached to the comment, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", '' — Type of media attachment: 'photo' — photo, 'video' — video, 'audio' — audio, 'doc' — document, '<owner_id>' — ID of the media attachment owner. '<media_id>' — Media attachment ID. Example: "photo100172_166443618,photo66748_265827614"
        """
    
        if attachments:
            attachments = ','.join(attachments)
        
        method_name = 'video.editComment'
        return self._call(method_name, locals())

    def get(self, owner_id: int = None, videos: List[str] = None, album_id: int = None, count: int = None, offset: int = None, extended: bool = None):
        """
        Returns detailed information about videos.
        
        :param owner_id: ID of the user or community that owns the video(s).
        :param videos: Video IDs, in the following format: "<owner_id>_<media_id>,<owner_id>_<media_id>", Use a negative value to designate a community ID. Example: "-4363_136089719,13245770_137352259"
        :param album_id: ID of the album containing the video(s).
        :param count: Number of videos to return.
        :param offset: Offset needed to return a specific subset of videos.
        :param extended: '1' — to return an extended response with additional fields
        """
    
        if videos:
            videos = ','.join(videos)
        
        method_name = 'video.get'
        return self._call(method_name, locals())

    def get_album_by_id(self, album_id: int, owner_id: int = None):
        """
        Returns video album info
        
        :param album_id: Album ID.
        :param owner_id: identifier of a user or community to add a video to. Use a negative value to designate a community ID.
        """
    
        method_name = 'video.getAlbumById'
        return self._call(method_name, locals())

    def get_albums(self, owner_id: int = None, offset: int = None, count: int = None, extended: bool = None, need_system: bool = None):
        """
        Returns a list of video albums owned by a user or community.
        
        :param owner_id: ID of the user or community that owns the video album(s).
        :param offset: Offset needed to return a specific subset of video albums.
        :param count: Number of video albums to return.
        :param extended: '1' — to return additional information about album privacy settings for the current user
        :param need_system: 
        """
    
        method_name = 'video.getAlbums'
        return self._call(method_name, locals())

    def get_albums_by_video(self, owner_id: int, video_id: int, target_id: int = None, extended: bool = None):
        """
        
        :param owner_id: 
        :param video_id: 
        :param target_id: 
        :param extended: 
        """
    
        method_name = 'video.getAlbumsByVideo'
        return self._call(method_name, locals())

    def get_comments(self, video_id: int, owner_id: int = None, need_likes: bool = None, start_comment_id: int = None, offset: int = None, count: int = None, sort: str = None, extended: bool = None, fields: List[str] = None):
        """
        Returns a list of comments on a video.
        
        :param video_id: Video ID.
        :param owner_id: ID of the user or community that owns the video.
        :param need_likes: '1' — to return an additional 'likes' field
        :param start_comment_id: 
        :param offset: Offset needed to return a specific subset of comments.
        :param count: Number of comments to return.
        :param sort: Sort order: 'asc' — oldest comment first, 'desc' — newest comment first
        :param extended: 
        :param fields: 
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'video.getComments'
        return self._call(method_name, locals())

    def remove_from_album(self, owner_id: int, video_id: int, target_id: int = None, album_id: int = None, album_ids: List[int] = None):
        """
        
        :param owner_id: 
        :param video_id: 
        :param target_id: 
        :param album_id: 
        :param album_ids: 
        """
    
        if album_ids:
            album_ids = [str(a) for a in album_ids]
            album_ids = ','.join(album_ids)
        
        method_name = 'video.removeFromAlbum'
        return self._call(method_name, locals())

    def reorder_albums(self, album_id: int, owner_id: int = None, before: int = None, after: int = None):
        """
        Reorders the album in the list of user video albums.
        
        :param album_id: Album ID.
        :param owner_id: ID of the user or community that owns the albums..
        :param before: ID of the album before which the album in question shall be placed.
        :param after: ID of the album after which the album in question shall be placed.
        """
    
        method_name = 'video.reorderAlbums'
        return self._call(method_name, locals())

    def reorder_videos(self, owner_id: int, video_id: int, target_id: int = None, album_id: int = None, before_owner_id: int = None, before_video_id: int = None, after_owner_id: int = None, after_video_id: int = None):
        """
        Reorders the video in the video album.
        
        :param owner_id: ID of the user or community that owns the video.
        :param video_id: ID of the video.
        :param target_id: ID of the user or community that owns the album with videos.
        :param album_id: ID of the video album.
        :param before_owner_id: ID of the user or community that owns the video before which the video in question shall be placed.
        :param before_video_id: ID of the video before which the video in question shall be placed.
        :param after_owner_id: ID of the user or community that owns the video after which the photo in question shall be placed.
        :param after_video_id: ID of the video after which the photo in question shall be placed.
        """
    
        method_name = 'video.reorderVideos'
        return self._call(method_name, locals())

    def report(self, owner_id: int, video_id: int, reason: int = None, comment: str = None, search_query: str = None):
        """
        Reports (submits a complaint about) a video.
        
        :param owner_id: ID of the user or community that owns the video.
        :param video_id: Video ID.
        :param reason: Reason for the complaint: '0' – spam, '1' – child pornography, '2' – extremism, '3' – violence, '4' – drug propaganda, '5' – adult material, '6' – insult, abuse
        :param comment: Comment describing the complaint.
        :param search_query: (If the video was found in search results.) Search query string.
        """
    
        method_name = 'video.report'
        return self._call(method_name, locals())

    def report_comment(self, owner_id: int, comment_id: int, reason: int = None):
        """
        Reports (submits a complaint about) a comment on a video.
        
        :param owner_id: ID of the user or community that owns the video.
        :param comment_id: ID of the comment being reported.
        :param reason: Reason for the complaint: , 0 – spam , 1 – child pornography , 2 – extremism , 3 – violence , 4 – drug propaganda , 5 – adult material , 6 – insult, abuse
        """
    
        method_name = 'video.reportComment'
        return self._call(method_name, locals())

    def restore(self, video_id: int, owner_id: int = None):
        """
        Restores a previously deleted video.
        
        :param video_id: Video ID.
        :param owner_id: ID of the user or community that owns the video.
        """
    
        method_name = 'video.restore'
        return self._call(method_name, locals())

    def restore_comment(self, comment_id: int, owner_id: int = None):
        """
        Restores a previously deleted comment on a video.
        
        :param comment_id: ID of the deleted comment.
        :param owner_id: ID of the user or community that owns the video.
        """
    
        method_name = 'video.restoreComment'
        return self._call(method_name, locals())

    def save(self, name: str = None, description: str = None, is_private: bool = None, wallpost: bool = None, link: str = None, group_id: int = None, album_id: int = None, privacy_view: List[str] = None, privacy_comment: List[str] = None, no_comments: bool = None, repeat: bool = None, compression: bool = None):
        """
        Returns a server address (required for upload) and video data.
        
        :param name: Name of the video.
        :param description: Description of the video.
        :param is_private: '1' — to designate the video as private (send it via a private message), the video will not appear on the user's video list and will not be available by ID for other users, '0' — not to designate the video as private
        :param wallpost: '1' — to post the saved video on a user's wall, '0' — not to post the saved video on a user's wall
        :param link: URL for embedding the video from an external website.
        :param group_id: ID of the community in which the video will be saved. By default, the current user's page.
        :param album_id: ID of the album to which the saved video will be added.
        :param privacy_view: 
        :param privacy_comment: 
        :param no_comments: 
        :param repeat: '1' — to repeat the playback of the video, '0' — to play the video once,
        :param compression: 
        """
    
        if privacy_view:
            privacy_view = ','.join(privacy_view)
        if privacy_comment:
            privacy_comment = ','.join(privacy_comment)
        
        method_name = 'video.save'
        return self._call(method_name, locals())

    def search(self, q: str, sort: int = None, hd: int = None, adult: bool = None, filters: List[str] = None, search_own: bool = None, offset: int = None, longer: int = None, shorter: int = None, count: int = None, extended: bool = None):
        """
        Returns a list of videos under the set search criterion.
        
        :param q: Search query string (e.g., 'The Beatles').
        :param sort: Sort order: '1' — by duration, '2' — by relevance, '0' — by date added
        :param hd: If not null, only searches for high-definition videos.
        :param adult: '1' — to disable the Safe Search filter, '0' — to enable the Safe Search filter
        :param filters: Filters to apply: 'youtube' — return YouTube videos only, 'vimeo' — return Vimeo videos only, 'short' — return short videos only, 'long' — return long videos only
        :param search_own: 
        :param offset: Offset needed to return a specific subset of videos.
        :param longer: 
        :param shorter: 
        :param count: Number of videos to return.
        :param extended: 
        """
    
        if filters:
            filters = ','.join(filters)
        
        method_name = 'video.search'
        return self._call(method_name, locals())




from ._ApiMethod import ApiMethod
from typing import List


class Streaming(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def get_server_url(self):
        """
        Allows to receive data for the connection to Streaming API.
        
        """
    
        method_name = 'streaming.getServerUrl'
        return self._call(method_name, locals())

    def set_settings(self, monthly_tier: str = None):
        """
        
        :param monthly_tier: 
        """
    
        method_name = 'streaming.setSettings'
        return self._call(method_name, locals())




import asyncio
import aiohttp
import time
import math

from typing import List


def get_items_packs(items: list, pack_size: int):
    items_packs = [items[i:i + pack_size] for i in range(0, len(items), pack_size)]
    return items_packs


class Parser:
    def __init__(self, access_tokens: List[str]):
        self._access_tokens = access_tokens
        self.current_response = None

    async def _execute(self, code: str, access_token: str):
        time_start = time.time()

        url = 'https://api.vk.com/method/execute'
        params = {'access_token': access_token, 'v': '5.120', 'code': code}
        async with aiohttp.ClientSession() as session:
            response = await session.post(url, params=params)
            response = await response.json()

        delay = 1 / 2.8 - (time.time() - time_start)
        if delay > 0:
            time.sleep(delay)
        return response['response']

    async def _parse_single_pack(self, items_pack: list, parse_method, access_token: str, ):
        code = """
        var items = {};
        var i = 0;
        var result = [];
        
        while (i < 25 && i < items.length) {{
            var r = {};
            result = result + r;
            i = i + 1;
        }}
        return result;
        """.format(items_pack, parse_method)
        response = await self._execute(code, access_token)
        return response

    async def _parse_items(self, items: list, pack_size: int, parse_method: str, access_token: str):
        result = []
        items_packs = get_items_packs(items, pack_size)
        for items_pack in items_packs:
            r = await self._parse_single_pack(items_pack, parse_method, access_token)
            result.extend(r)
        return result

    def fast_items_parse(self, items: list, parse_method: str, pack_size: int = 25):
        items_per_acc = math.ceil(len(items) / len(self._access_tokens))
        accounts_items = get_items_packs(items, items_per_acc)

        loop = asyncio.new_event_loop()
        tasks = [
            loop.create_task(
                self._parse_items(ai, pack_size, parse_method, at)
            ) for ai, at in zip(accounts_items, self._access_tokens)
        ]

        loop.run_until_complete(asyncio.wait(tasks))
        loop.close()

        result = []
        for t in tasks:
            result.extend(t.result())
        return result



from ._ApiMethod import ApiMethod
from typing import List


class Users(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def get(self, user_ids: List[str] = None, fields: List[str] = None, name_case: str = None):
        """
        Returns detailed information on users.
        
        :param user_ids: User IDs or screen names ('screen_name'). By default, current user ID.
        :param fields: Profile fields to return. Sample values: 'nickname', 'screen_name', 'sex', 'bdate' (birthdate), 'city', 'country', 'timezone', 'photo', 'photo_medium', 'photo_big', 'has_mobile', 'contacts', 'education', 'online', 'counters', 'relation', 'last_seen', 'activity', 'can_write_private_message', 'can_see_all_posts', 'can_post', 'universities', 'can_invite_to_chats'
        :param name_case: Case for declension of user name and surname: 'nom' — nominative (default), 'gen' — genitive , 'dat' — dative, 'acc' — accusative , 'ins' — instrumental , 'abl' — prepositional
        """
    
        if user_ids:
            user_ids = ','.join(user_ids)
        if fields:
            fields = ','.join(fields)
        
        method_name = 'users.get'
        return self._call(method_name, locals())

    def get_followers(self, user_id: int = None, offset: int = None, count: int = None, fields: List[str] = None, name_case: str = None):
        """
        Returns a list of IDs of followers of the user in question, sorted by date added, most recent first.
        
        :param user_id: User ID.
        :param offset: Offset needed to return a specific subset of followers.
        :param count: Number of followers to return.
        :param fields: Profile fields to return. Sample values: 'nickname', 'screen_name', 'sex', 'bdate' (birthdate), 'city', 'country', 'timezone', 'photo', 'photo_medium', 'photo_big', 'has_mobile', 'rate', 'contacts', 'education', 'online'.
        :param name_case: Case for declension of user name and surname: 'nom' — nominative (default), 'gen' — genitive , 'dat' — dative, 'acc' — accusative , 'ins' — instrumental , 'abl' — prepositional
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'users.getFollowers'
        return self._call(method_name, locals())

    def get_subscriptions(self, user_id: int = None, extended: bool = None, offset: int = None, count: int = None, fields: List[str] = None):
        """
        Returns a list of IDs of users and communities followed by the user.
        
        :param user_id: User ID.
        :param extended: '1' — to return a combined list of users and communities, '0' — to return separate lists of users and communities (default)
        :param offset: Offset needed to return a specific subset of subscriptions.
        :param count: Number of users and communities to return.
        :param fields: 
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'users.getSubscriptions'
        return self._call(method_name, locals())

    def report(self, user_id: int, type_: str, comment: str = None):
        """
        Reports (submits a complain about) a user.
        
        :param user_id: ID of the user about whom a complaint is being made.
        :param type_: Type of complaint: 'porn' – pornography, 'spam' – spamming, 'insult' – abusive behavior, 'advertisement' – disruptive advertisements
        :param comment: Comment describing the complaint.
        """
    
        param_alias_dict = {'type_': 'type'}
        method_name = 'users.report'
        return self._call(method_name, locals())

    def search(self, q: str = None, sort: int = None, offset: int = None, count: int = None, fields: List[str] = None, city: int = None, country: int = None, hometown: str = None, university_country: int = None, university: int = None, university_year: int = None, university_faculty: int = None, university_chair: int = None, sex: int = None, status: int = None, age_from: int = None, age_to: int = None, birth_day: int = None, birth_month: int = None, birth_year: int = None, online: bool = None, has_photo: bool = None, school_country: int = None, school_city: int = None, school_class: int = None, school: int = None, school_year: int = None, religion: str = None, company: str = None, position: str = None, group_id: int = None, from_list: List[str] = None):
        """
        Returns a list of users matching the search criteria.
        
        :param q: Search query string (e.g., 'Vasya Babich').
        :param sort: Sort order: '1' — by date registered, '0' — by rating
        :param offset: Offset needed to return a specific subset of users.
        :param count: Number of users to return.
        :param fields: Profile fields to return. Sample values: 'nickname', 'screen_name', 'sex', 'bdate' (birthdate), 'city', 'country', 'timezone', 'photo', 'photo_medium', 'photo_big', 'has_mobile', 'rate', 'contacts', 'education', 'online',
        :param city: City ID.
        :param country: Country ID.
        :param hometown: City name in a string.
        :param university_country: ID of the country where the user graduated.
        :param university: ID of the institution of higher education.
        :param university_year: Year of graduation from an institution of higher education.
        :param university_faculty: Faculty ID.
        :param university_chair: Chair ID.
        :param sex: '1' — female, '2' — male, '0' — any (default)
        :param status: Relationship status: '1' — Not married, '2' — In a relationship, '3' — Engaged, '4' — Married, '5' — It's complicated, '6' — Actively searching, '7' — In love
        :param age_from: Minimum age.
        :param age_to: Maximum age.
        :param birth_day: Day of birth.
        :param birth_month: Month of birth.
        :param birth_year: Year of birth.
        :param online: '1' — online only, '0' — all users
        :param has_photo: '1' — with photo only, '0' — all users
        :param school_country: ID of the country where users finished school.
        :param school_city: ID of the city where users finished school.
        :param school_class: 
        :param school: ID of the school.
        :param school_year: School graduation year.
        :param religion: Users' religious affiliation.
        :param company: Name of the company where users work.
        :param position: Job position.
        :param group_id: ID of a community to search in communities.
        :param from_list: 
        """
    
        if fields:
            fields = ','.join(fields)
        if from_list:
            from_list = ','.join(from_list)
        
        method_name = 'users.search'
        return self._call(method_name, locals())




import time
from ..exceptions  import raise_exception, Server


class ApiMethod:
    def __init__(self, session, access_token, v, requests_per_s):
        self._session = session
        self._access_token = access_token
        self._v = v
        self._requests_per_s = requests_per_s

    def _call(self, method_name, method_locals):
        try:
            time_start = time.time()

            api_url = f'https://api.vk.com/method/{method_name}'

            params = method_locals
            param_alias_dict = params.get('param_alias_dict')
            if param_alias_dict:
                for alias in param_alias_dict:
                    params[param_alias_dict[alias]] = params.pop(alias)
                params.pop('param_alias_dict')

            params.pop('self')
            params = {p: params[p] for p in params if params[p] is not None}
            params['access_token'] = self._access_token
            params['v'] = self._v

            response = self._session.post(url=api_url, params=params).json()
            if 'error' in response:
                # print(response)
                error_code =  response['error']['error_code']
                error_message = response['error']['error_msg']
                raise_exception(error_code, error_message)
            else:
                response = response['response']

            delay = 1 / self._requests_per_s - (time.time() - time_start)
            if delay > 0:
                time.sleep(delay)

        except Server as e:
            response = self._call(method_name, method_locals)


        return response



from ._ApiMethod import ApiMethod
from typing import List


class Widgets(ApiMethod):
    def __init__(self, session, access_token, v, requests_per_s):
        super().__init__(session, access_token, v, requests_per_s)

    def get_comments(self, widget_api_id: int = None, url: str = None, page_id: str = None, order: str = None, fields: List[str] = None, offset: int = None, count: int = None):
        """
        Gets a list of comments for the page added through the [vk.com/dev/Comments|Comments widget].
        
        :param widget_api_id: 
        :param url: 
        :param page_id: 
        :param order: 
        :param fields: 
        :param offset: 
        :param count: 
        """
    
        if fields:
            fields = ','.join(fields)
        
        method_name = 'widgets.getComments'
        return self._call(method_name, locals())

    def get_pages(self, widget_api_id: int = None, order: str = None, period: str = None, offset: int = None, count: int = None):
        """
        Gets a list of application/site pages where the [vk.com/dev/Comments|Comments widget] or [vk.com/dev/Like|Like widget] is installed.
        
        :param widget_api_id: 
        :param order: 
        :param period: 
        :param offset: 
        :param count: 
        """
    
        method_name = 'widgets.getPages'
        return self._call(method_name, locals())

